import justpy as jp

grid_options = """
{
    defaultColDef: {
        filter: true,
        sortable: true,
        resizable: true,
        cellStyle: {textAlign: 'center'},
        headerClass: 'font-bold'
    },
      columnDefs: [
      {headerName: "Make", field: "make"},
      {headerName: "Model", field: "model"},
      {headerName: "Price", field: "price"}
    ],
      rowData: [
      {make: "Toyota", model: "Celica", price: 35000},
      {make: "Ford", model: "Mondeo", price: 32000},
      {make: "Porsche", model: "Boxter", price: 72000}
    ]
}
"""

class Issue50:
    """
    Issue 50 example code
    """
    def cell_changed(self, msg):
        """
        react on a changed cell value
        """
        print(msg)
        change_msg=f"'Old value: {msg.oldValue}, New value: {msg.newValue}, Row Index: {msg.rowIndex}, Column ID: {msg.colId}'"
        self.feedback.text=change_msg
        print(change_msg)
        print(f'Row data: {msg.data}')
        # Here you would update your database or Pandas frame etc.
    
    def grid_test(self):
        """
        WebPage to test justpy issue 50
        """
        wp = jp.WebPage()
        self.feedback=jp.Div(a=wp,text="edit a grid cell ...")
        grid = jp.AgGrid(a=wp, options=grid_options, style='height: 200px; width: 300px; margin: 0.25em')
        for col_def in grid.options.columnDefs:
            col_def.editable = True  # You can add this directly to the columnDefs in grid_options
        grid.on('cellValueChanged', self.cell_changed)
        return wp

def issue50_grid_test():
    """
    object oriented wrapper
    """
    issue50=Issue50()
    return issue50.grid_test()

# make me available on the command line and for the demo_browser
from examples.basedemo import Demo
Demo("Issue 50 aggrid cell value changed", issue50_grid_test)



# Justpy Tutorial demo todo_app4 from docs/blog/reactivity.md
import justpy as jp


class TodoList(jp.Div):

    def __init__(self, **kwargs):
        self.todos = []
        super().__init__(**kwargs)
        self.ol = jp.Ol(a=self)
        self.task = jp.Input(placeholder="Enter task", a=self)
        jp.Button(
            text="Add Task", a=self, click=self.add_task, style="margin-left: 10px"
        )

    def add_task(self, msg):
        self.todos.append(self.task.value)
        self.task.value = ""

    def react(self, data):
        self.ol.delete_components()
        for todo in self.todos:
            jp.Li(text=todo, a=self.ol)


def todo_app4():
    wp = jp.WebPage(tailwind=False)
    TodoList(a=wp, todos=["Buy groceries", "Learn JustPy", "Do errands"])
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("todo_app4", todo_app4)



# Justpy Tutorial demo loading_page_no_stagger from docs/tutorial/page_events.md
import justpy as jp
import asyncio


async def page_ready_div(self, msg):
    for i in range(1, 3001):
        jp.Div(text=f"Div {i}", a=self.d, classes="border m-2 p-2 text-xs")
        if i % 100 == 0:
            await self.update()
            await asyncio.sleep(0.25)


@jp.SetRoute("/stagger")
def loading_page_stagger_test():
    wp = jp.WebPage()
    wp.on("page_ready", page_ready_div)
    wp.d = jp.Div(classes="flex flex-wrap", a=wp)
    return wp


def loading_page_no_stagger():
    wp = jp.WebPage()
    wp.d = jp.Div(classes="flex flex-wrap", a=wp)
    for i in range(1, 3001):
        jp.Div(text=f"Div {i}", a=wp.d, classes="border m-2 p-2 text-xs")
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("loading_page_no_stagger", loading_page_no_stagger)



"""
Created on 2022-09-05

@author: wf
"""
from tests.base_server_test import BaseAsynctest
from tests.basetest import Basetest
from jpcore.demostarter import Demostarter
import unittest
import asyncio
import pprint

class TestDemoStarter(BaseAsynctest):
    """
    test demo starter
    """

    async def setUp(self):
        await BaseAsynctest.setUp(self, with_server=False)
        
    def test_videos(self):
        """
        test video availability
        """
        demo_starter = Demostarter(debug=True, mode="process")
        debug=True
        foundVideos=0
        for demo in demo_starter.demos:
            if demo.video_url is not None:
                foundVideos+=1
        if debug:
            print(f"found {foundVideos} videos")
        self.assertTrue(foundVideos>=30)
        
    def test_example_sources(self):
        """
        test example source handling
        """
        demo_starter = Demostarter()
        debug=self.debug
        debug=True
        for demo in demo_starter.demos:
            if debug:
                print(demo)

    @unittest.skipIf(Basetest.inPublicCI(), "demostarter ")
    async def test_demo_starter(self):
        """
        test the demo starter
        """
        #@TODO reenable test
        return 
        demoStarter = Demostarter(debug=True, mode="process")

        await demoStarter.start(limit=2)

        # wait a bit
        await asyncio.sleep(2.0)
        # stop all servers
        await demoStarter.stop()
        with self.subTest("testDemoStarter", demoStarter=demoStarter):
            self.assertEqual(0, len(demoStarter.errors.values()))



# Justpy Tutorial demo hello_test from docs/tutorial/ajax.md
import justpy as jp

def hello_test():
    wp = jp.WebPage()
    for i in range(10):
        jp.Hello(a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("hello_test",hello_test, websockets=False)



import justpy as jp
import pandas as pd

iris = pd.read_csv(
    "https://raw.githubusercontent.com/mwaskom/seaborn-data/master/iris.csv"
)
iris_stats = iris.describe().round(3)
iris_stats.insert(loc=0, column="stats", value=iris_stats.index)
iris_species = list(iris["species"].unique())

# Create a dictionary of frames per iris species
iris_species_frames = {}
for s in iris_species:
    iris_species_frames[s] = iris.loc[iris["species"] == s]


async def click_point(self, msg):
    # print(msg)
    return await self.select_point(
        [
            {"id": chart_id, "series": msg.series_index, "point": msg.point_index}
            for chart_id in self.chart_list
            if self.id != chart_id
        ],
        msg.websocket,
    )


async def tooltip_formatter(self, msg):
    # print(msg)
    tooltip_html = f"""
    <div style="color: {msg.color}"><span>&#x25CF;</span> {msg.series_name}</div>
    <div style="color: {msg.color}">{self.col1}: {msg.x}</div>
    <div style="color: {msg.color}">{self.col2}: {msg.y}</div>
    """
    await self.draw_crosshair(
        [
            {"id": chart_id, "series": msg.series_index, "point": msg.point_index}
            for chart_id in self.chart_list
        ],
        msg.websocket,
    )
    return await self.tooltip_update(tooltip_html, msg.websocket)


def iris_data():
    wp = jp.WebPage(highcharts_theme="gray", title="Iris Dataset", debug=True)
    jp.Div(
        text="Iris Dataset",
        classes="text-3xl m-2 p-2 font-medium tracking-wider text-yellow-300 bg-gray-800 text-center",
        a=wp,
    )
    d1 = jp.Div(classes="m-2 p-2 border-2", a=wp)
    chart_list = []
    for i, col1 in enumerate(iris.columns[:4]):
        d2 = jp.Div(classes="flex", a=d1)
        for j, col2 in enumerate(iris.columns[:4]):
            if i != j:  # Not on the diagonal
                chart = jp.HighCharts(
                    a=d2, style="width: 300px; height: 300px", classes="flex-grow m-1"
                )
                chart_list.append(chart.id)
                chart.chart_list = chart_list
                chart.on("tooltip", tooltip_formatter)
                chart.tooltip_y = 85
                chart.on("point_click", click_point)
                chart.col1 = col1
                chart.col2 = col2
                o = chart.options
                o.chart.type = "scatter"
                o.chart.zoomType = "xy"
                o.title.text = ""
                o.legend.enabled = False
                o.credits.enabled = (
                    False if i < 3 or j < 3 else True
                )  # https://api.highcharts.com/highcharts/credits.enabled
                o.xAxis.title.text = col2 if i == 3 else ""
                o.yAxis.title.text = col1 if j == 0 else ""
                o.xAxis.crosshair = o.yAxis.crosshair = True
                for k, v in iris_species_frames.items():
                    s = jp.Dict()
                    s.name = k
                    s.allowPointSelect = True  # https://api.highcharts.com/highcharts/series.scatter.allowPointSelect
                    s.marker.states.select.radius = 8
                    s.data = list(zip(v.iloc[:, j], v.iloc[:, i]))
                    o.series.append(s)
            else:
                chart = jp.Histogram(
                    list(iris.iloc[:, j]),
                    a=d2,
                    style="width: 300px; height: 300px",
                    classes="flex-grow m-1",
                )
                o = chart.options
                o.title.text = ""
                o.legend.enabled = False
                o.xAxis[0].title.text = col2 if i == 3 else ""
                o.xAxis[1].title.text = ""
                o.yAxis[0].title.text = col1 if j == 0 else ""
                o.yAxis[1].title.text = ""
                o.credits.enabled = False if i < 3 or j < 3 else True

    # Add two grids, first with the data and second with statistics describing the data
    iris.jp.ag_grid(
        a=wp,
        classes="m-2 p-2",
        style="height: 500px; width: 800px",
        auto_size=True,
        theme="ag-theme-balham-dark",
    )
    iris_stats.jp.ag_grid(
        a=wp,
        classes="m-2 p-2 border",
        style="height: 500px; width: 950px",
        auto_size=True,
        theme="ag-theme-material",
    )
    return wp


from examples.basedemo import Demo

Demo("iris demo", iris_data, HIGHCHARTS=True, AGGRID=True)



# Justpy Tutorial demo html_demo from docs/tutorial/working_with_html.md
import justpy as jp

def html_demo():
    wp = jp.WebPage()
    jp.Div(text='This will not be shown', a=wp)
    wp.html = '<p class="text-2xl m-2 m-1 text-red-500">Hello world!<p>'
    jp.Div(text='This will not be shown', a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("html_demo",html_demo)



# Justpy Tutorial demo eq_test from docs/tutorial/equations.md
import justpy as jp

integral = r"""
\f\relax{x} = \int_{-\infty}^\infty \f\hat\xi\,e^{2 \pi i \xi x} \,d\xi
"""

f1 = r"""
     a + ar + a r^2 + a r^3 + \cdots + a r^{n-1} =
     \sum_{k=0}^{n-1} ar^k =
     a \left(\frac{1-r^{n}}{1-r}\right)
     """

f2 = r"""
\displaystyle \frac{1}{\Bigl(\sqrt{\phi \sqrt{5}}-\phi\Bigr) e^{\frac25 \pi}} = 1+\frac{e^{-2\pi}} {1+\frac{e^{-4\pi}} {1+\frac{e^{-6\pi}} {1+\frac{e^{-8\pi}} {1+\cdots} } } }
"""


async def button_click(self, msg):
    wp = msg.page
    wp.eq1.equation += " +  \\sqrt{d} + 1"


def eq_test(request):
    wp = jp.WebPage()
    wp.btn = jp.Button(
        text="Modify Equation",
        click=button_click,
        a=wp,
        classes=jp.Styles.button_simple + " m-2 p-2",
    )
    wp.eq1 = jp.Equation(
        equation="c = \\pm\\sqrt{a^2 + b^2 + c^2}", classes="text-3xl m-2 p-2", a=wp
    )
    # Add a macro
    eq2 = jp.Equation(equation=integral, classes="text-3xl m-2 p-2", a=wp)
    eq2.options["macros"]["\\f"] = "#1f(#2)"
    jp.Equation(equation=f1, classes="text-3xl m-2 p-2", a=wp)
    jp.Equation(equation=f2, classes="text-3xl m-2 p-2", a=wp)
    # Use raw strings when convenient
    jp.Equation(
        equation="c = \\pm\\sqrt{a^2 + b^2}\\in\\RR",
        options={"macros": {"\\RR": "\\mathbb{R}"}},
        classes="text-3xl m-2 p-2",
        a=wp,
    )
    jp.Equation(
        equation=r"e = \pm\sqrt{c^2 + d^2}\in\RR",
        options={"macros": {"\\RR": "\\mathbb{R}"}},
        classes="text-3xl m-2 p-2",
        a=wp,
    )
    # You can also add equations using the WebPage equation method
    wp.equation(r"e^{i\pi}+1=0")
    a = wp.equation(r"e^{i\pi}+1=0")
    a.classes = "bg-blue-500 text-5xl text-white m-2 p-4 inline-block"
    return wp


# You can also add KATEX = True to justpy.env instead of specifying below
# initialize the demo
from examples.basedemo import Demo

Demo("eq_test", eq_test, KATEX=True)



'''
Created on 2022-09-14

@author: wf
'''
import asyncio
from selenium.webdriver.common.by import By
from tests.base_selenium_test import BaseSeleniumTest
from jpcore.webpage import WebPage

class TestAjaxWithSelenium(BaseSeleniumTest):
    """
    testing ajax (non-websocket) behavior
    """

    async def setUp(self):
        await super().setUp(port=8126)
        
    async def testAjax(self):
        """
        test ajax (non-websocket behavior) according to tutorial
        """
        WebPage.use_websockets=False
        self.browser = await self.getBrowserForDemo()
        from examples.tutorial.ajax.hello_test import hello_test
        await self.server.start(hello_test,websockets=False)
        url = self.server.get_url("/")
        self.browser.get(url)
        await asyncio.sleep(self.server.sleep_time)
        divs = self.browser.find_elements(By.TAG_NAME, "div")
        for div in divs:
            div.click()
            await asyncio.sleep(0.1)
        await asyncio.sleep(self.server.sleep_time)
        #debug=self.debug
        debug=True
        for div in divs:
            if debug:
                print(div.text)
            self.assertTrue(f"was clicked 1 times" in div.text)
        self.browser.close()
        await asyncio.sleep(self.server.sleep_time)
        await self.server.stop()


# Justpy Tutorial demo grid_test12 from docs/grids_tutorial/grid_events.md
import justpy as jp
import pandas as pd

wm_df = pd.read_csv('https://elimintz.github.io/women_majors.csv').round(2)

def row_selected3(self, msg):
    print(msg)
    if msg.selected:
        categories, values = list(msg.data.keys()), list(msg.data.values())
        o = self.row_chart.options
        o.title.text = f'{values[0]} - Percent Women in Major'
        o.xAxis.categories = categories[1:]
        s = jp.Dict({'name': values[0], 'data': values[1:]})
        s.dataLabels.enabled = True
        if len(o.series) > 0:
            o.series[0] = s
        else:
            o.series.append(s)
        self.row_selected3 = msg.rowIndex
        self.row_chart.show = True
    elif self.row_selected3 == msg.rowIndex:
        self.row_chart.show = False

def grid_test12():
    wp = jp.WebPage()
    row_chart = jp.HighCharts(a=wp)
    row_chart.options.chart.type = 'column'
    row_chart.options.yAxis.title.text = '%'
    row_chart.show = False
    grid = wm_df.jp.ag_grid(a=wp)
    grid.row_chart = row_chart
    grid.on('rowSelected', row_selected3)
    grid.options.columnDefs[0].checkboxSelection = True
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("grid_test12",grid_test12)



from .htmlcomponents import *
from .chartcomponents import *
from .gridcomponents import *
from addict import Dict
from io import StringIO

try:
    import pandas as pd

    _has_pandas = True
except:
    _has_pandas = False


# https://pandas.pydata.org/pandas-docs/stable/development/extending.html
if _has_pandas:

    @pd.api.extensions.register_dataframe_accessor("jp")
    class JustPyAccessor:
        def __init__(self, df, **kwargs):
            self._validate(df)
            self.df = df

        @staticmethod
        def _validate(obj):
            pass

        @staticmethod
        def make_pairs_list(x_data, y_data):
            return list(map(list, itertools.zip_longest(x_data, y_data)))

        def _get_column(self, col_spec):
            if isinstance(col_spec, int):
                col = self.df.iloc[:, col_spec]
            elif isinstance(col_spec, str):
                col = self.df[col_spec]
            else:
                raise TypeError(
                    "Column specification for plotting must be integer or string"
                )
            col = col.replace(
                [np.inf, -np.inf], [sys.float_info.max, -sys.float_info.max]
            )
            # Convert nan to None
            return col.where((pd.notnull(col)), None)

        def plot(self, x, y, **kwargs):
            kind = kwargs.get("kind", "column")
            chart = HighCharts(**kwargs)
            categories = kwargs.get("categories", True)
            o = chart.options
            o.chart.type = kind
            o.chart.zoomType = "xy"
            o.chart.panning = True
            o.chart.panKey = "shift"
            o.title.text = kwargs.get("title", "")
            o.subtitle.text = kwargs.get("subtitle", "")
            o.plotOptions.series.stacking = kwargs.get(
                "stacking", ""
            )  # either normal or percent
            if kind not in ["scatter"] and categories:
                o.xAxis.categories = list(self._get_column(x))
            o.series = []
            for col in y:
                s = Dict()
                if kind not in ["scatter"] and categories:
                    s.data = list(self._get_column(col))
                else:
                    s.data = self.make_pairs_list(
                        self._get_column(x), self._get_column(col)
                    )
                s.name = self.df.columns[col] if isinstance(col, int) else col
                s.type = kind
                o.series.append(s)
            return chart

        def ag_grid(self, **kwargs):
            grid = AgGrid(**kwargs)
            grid.load_pandas_frame(self.df)
            return grid

        def table(self, **kwargs):
            headers = list(self.df.columns)
            table_data = self.df.to_numpy().tolist()
            table_data.insert(0, headers)
            return AutoTable(values=table_data, **kwargs)

    def read_csv_from_string(csv_string, *args):
        return pd.read_csv(StringIO(csv_string), *args)

    class LinkedChartGrid(Div):
        def __init__(self, df, x, y, **kwargs):
            super().__init__(**kwargs)
            self.df = df
            self.x = x
            self.y = y
            self.kind = kwargs.get("kind", "column")
            self.stacking = kwargs.get("stacking", "")
            self.title = kwargs.get("title", "")
            self.subtitle = kwargs.get("subtitle", "")
            self.set_classes("flex flex-col")
            self.chart = df.jp.plot(
                x,
                y,
                a=self,
                classes="m-2 p-2 border",
                kind=self.kind,
                stacking=self.stacking,
                title=self.title,
                subtitle=self.subtitle,
            )
            self.grid = df.jp.ag_grid(a=self)
            self.grid.parent = self
            for event_name in [
                "sortChanged",
                "filterChanged",
                "columnMoved",
                "rowDragEnd",
            ]:
                self.grid.on(event_name, self.grid_change)

        @staticmethod
        def grid_change(self, msg):
            self.parent.df = read_csv_from_string(msg.data)
            c = self.parent.df.jp.plot(
                self.parent.x,
                self.parent.y,
                kind=self.parent.kind,
                title=self.parent.title,
                subtitle=self.parent.subtitle,
                stacking=self.parent.stacking,
            )
            self.parent.chart.options = c.options



# Justpy Tutorial demo hello_test3 from docs/tutorial/custom_components.md
import justpy as jp

def hello_test3():
    wp = jp.WebPage()
    for i in range(5):
        jp.Hello(a=wp, counter=100)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("hello_test3",hello_test3)



# Justpy Tutorial demo todo_app1 from docs/blog/reactivity.md
import justpy as jp

todos = ["Go shopping", "Learn JustPy", "Do errands"]


def todo_app1():
    wp = jp.WebPage(tailwind=False)
    ol = jp.Ol(a=wp)
    for todo in todos:
        jp.Li(text=todo, a=ol)
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("todo_app1", todo_app1)



# Justpy Tutorial demo tool_tip_demo1 from docs/charts_tutorial/tooltips.md
import justpy as jp

# https://www.highcharts.com/demo/line-basic
chart_def = """
{
 chart: {
        type: 'spline'
    },
    title: {
        text: ''
    },
    subtitle: {
        text: 'JustPy Tooltip Demo'
    },
    yAxis: {
        title: {
            text: 'Number of Employees'
        }
    },
    legend: {
        layout: 'vertical',
        align: 'right',
        verticalAlign: 'middle'
    },
    plotOptions: {
        series: {
            pointStart: 2010
        }
    },
    series: [{
        name: 'Installation',
        data: [43934, 52503, 57177, 69658, 97031, 119931, 137133, 154175]
    }, {
        name: 'Manufacturing',
        data: [24916, 24064, 29742, 29851, 32490, 30282, 38121, 40434]
    }, {
        name: 'Sales & Distribution',
        data: [11744, 17722, 16005, 19771, 20185, 24377, 32147, 39387]
    }, {
        name: 'Project Development',
        data: [null, null, 7988, 12169, 15112, 22452, 34400, 34227]
    }, {
        name: 'Other',
        data: [12908, 5948, 8105, 11248, 8989, 11816, 18274, 18111]
    }]
}
"""


def tool_tip_demo1():
    wp = jp.WebPage()
    jp.Div(text='Charts with default Highcharts tooltip', classes='m-2 p-2 text-xl bg-blue-500 text-white',a =wp)
    d = jp.Div(classes='flex flex-wrap ', a=wp)
    my_charts = []
    for i in range(3):
        my_charts.append(jp.HighCharts(a=d, classes='m-2 p-2 border', options=chart_def))
    my_charts[0].options.tooltip.shared = False
    my_charts[1].options.tooltip.shared = True
    my_charts[2].options.tooltip.split = True
    my_charts[0].options.title.text = 'Simple Tooltip'
    my_charts[1].options.title.text = 'Shared Tooltip'
    my_charts[2].options.title.text = 'Split Tooltip'
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("tool_tip_demo1",tool_tip_demo1)



# Justpy Tutorial demo women_majors3 
# generated by demoapp.update_from tutorial at 2022-09-21T08:55:48.678952+00:00 
# see https://github.com/justpy-org/justpy/blob/master/examples/charts_tutorial/pandas/women_majors3.py
# 
# Using Pandas without the Extension            
# see https://justpy.io/charts_tutorial/pandas#using-pandas-without-the-extension
import justpy as jp
import pandas as pd
import itertools


wm = pd.read_csv('https://elimintz.github.io/women_majors.csv').round(2)
wm_under_20 = list(
  wm.loc[0, wm.loc[0] < 20].index
) # Create list of majors which start under 20%


def make_pairs_list(x_data, y_data):
    return list(map(list, itertools.zip_longest(x_data, y_data)))


def women_majors3():
    wp = jp.WebPage(highcharts_theme='grid')
    wm_chart = jp.HighCharts(a=wp, classes='m-2 p-2 w-3/4')
    o = wm_chart.options  # Will save us some typing and make code cleaner
    o.title.text = 'The gender gap is transitory - even for extreme cases'
    o.title.align = 'left'
    o.xAxis.title.text = 'Year'
    o.xAxis.gridLineWidth = 1
    o.yAxis.title.text = '% Women in Major'
    o.yAxis.labels.format = '{value}%'
    o.legend.layout = 'proximate'
    o.legend.align = 'right'
    o.series = []
    x_data = wm.iloc[:, 0].tolist()
    for major in wm_under_20:
        s = jp.Dict()
        y_data = wm[major].tolist()
        s.data = make_pairs_list(x_data, y_data)
        s.name = major
        s.type = 'spline'
        o.series.append(s)
        s.marker.enabled = False
    return wp

# initialize the demo
from examples.basedemo import Demo
Demo("women_majors3", women_majors3)



# Justpy Tutorial demo parse_demo1 from docs/tutorial/working_with_html.md
import justpy as jp

async def parse_demo1(request):
    wp = jp.WebPage()
    c = jp.parse_html("""
    <div>
    <p class="m-2 p-2 text-red-500 text-xl">Paragraph 1</p>
    <p class="m-2 p-2 text-blue-500 text-xl">Paragraph 2</p>
    <p class="m-2 p-2 text-green-500 text-xl">Paragraph 3</p>
    </div>
    """, a=wp)
    print(c)
    print(c.components)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("parse_demo1",parse_demo1)



# Justpy Tutorial demo stock_test1 from docs/charts_tutorial/stock_charts.md
import justpy as jp
import pandas as pd
import datetime

epoch = datetime.datetime(1970, 1, 1)


def convert_date1(date_string):
    date = datetime.datetime.strptime(date_string, "%Y-%m-%d")
    return (date - epoch).total_seconds() * 1000


def stock_test1(request):
    wp = jp.WebPage()
    ticker = request.query_params.get("ticker", "msft")
    if ticker not in ["aapl", "ibm", "intc", "msft"]:
        ticker = "msft"
    data = pd.read_csv(f"https://elimintz.github.io/stocks/{ticker.upper()}.csv")
    chart = jp.HighStock(a=wp, classes="m-1 p-2 border w-10/12")
    o = chart.options
    o.title.text = "Historical Stock Price"
    o.legend = {"enabled": True, "align": "right", "layout": "proximate"}
    o.rangeSelector.selected = 4  # Set default range to 1 year
    x = list(data["Date"].map(convert_date1))
    y = data["Adj Close"].to_list()
    s = jp.Dict({"name": ticker.upper(), "data": jp.make_pairs_list(x, y)})
    o.series = [s]
    s.tooltip.valueDecimals = 2  # Price displayed by tooltip will have 2 decimal values
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("stock_test1", stock_test1)



# Justpy Tutorial demo event_demo4 from docs/tutorial/handling_events.md
import justpy as jp

def my_click4(self, msg):
    self.text = 'I was clicked'
    self.set_class('bg-blue-500')

def my_mouseenter(self, msg):
    self.text = 'Mouse entered'
    self.set_class('bg-red-500')

def my_mouseleave(self, msg):
    self.text = 'Mouse left'
    self.set_class('bg-teal-500')


def event_demo4():
    wp = jp.WebPage()
    d = jp.Div(text='Not clicked yet', a=wp, classes='w-64 text-2xl m-2 p-2 bg-blue-500 text-white',
             click=my_click4, mouseenter=my_mouseenter, mouseleave=my_mouseleave)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("event_demo4",event_demo4)



# Justpy Tutorial demo quasar_tree_test from docs/quasar_tutorial/quasar_components.md
import justpy as jp

async def expand_tree(self, msg):
    return await self.tree.run_method('expandAll()', msg.websocket)

async def collapse_tree(self, msg):
    return await self.tree.run_method('collapseAll()', msg.websocket)


def quasar_tree_test():
    wp = jp.QuasarPage()
    d = jp.Div(classes="q-pa-md q-gutter-sm", a=wp)
    node_string = """
    [
        {
          label: 'Satisfied customers (with avatar)',
          avatar: 'https://cdn.quasar.dev/img/boy-avatar.png',
          children: [
            {
              label: 'Good food (with icon)',
              icon: 'restaurant_menu',
              children: [
                { label: 'Quality ingredients', icon: 'favorite' },
                { label: 'Good recipe' }
              ]
            },
            {
              label: 'Good service (disabled node with icon)',
              icon: 'room_service',
              disabled: true,
              children: [
                { label: 'Prompt attention' },
                { label: 'Professional waiter' }
              ]
            },
            {
              label: 'Pleasant surroundings (with icon)',
              icon: 'photo',
              children: [
                {
                  label: 'Happy atmosphere (with image)',
                  img: 'https://cdn.quasar.dev/img/logo_calendar_128px.png'
                },
                { label: 'Good table presentation' },
                { label: 'Pleasing decor' }
              ]
            }
          ]
        }
      ]
    """

    b1 = jp.QBtn(label='Expand', a=d, click=expand_tree)
    b2 = jp.QBtn(label='Collapse', a=d, click=collapse_tree)

    tree = jp.QTree(a=d, node_key='label', nodes=node_string, tick_strategy="leaf", no_connectors=False, default_expand_all=True)
    d1 = jp.Div(text='', a=d)
    
    def my_updated(self, msg):
        print('in my updated')
        d1.text = str(msg.value)
    tree.on('update:ticked', my_updated)
    
    b1.tree = tree
    b2.tree = tree
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("quasar_tree_test",quasar_tree_test)



# Justpy Tutorial demo debounce_test from docs/tutorial/handling_events.md
import justpy as jp

def mouse_event(self, msg):
    try:
        self.counter += 1
    except:
        self.counter = 1
    msg.page.info_div.add_first(jp.Div(text=f'{self.counter}) {msg.event_type}'))


def debounce_test():
    wp = jp.WebPage()
    d = jp.Div(style='height: 100vh', a=wp)
    d.add_event('mousemove')
    d.on('mousemove', mouse_event, throttle=1000)
    d.on('click', mouse_event, debounce=2000, immediate=False)
    wp.info_div = jp.Div(text='Recent mouse events', classes='m-4 text-lg', a=d)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("debounce_test",debounce_test)



# Justpy Tutorial demo grid_test11 from docs/grids_tutorial/grid_events.md
import justpy as jp
import pandas as pd

wm_df = pd.read_csv('https://elimintz.github.io/women_majors.csv').round(2)

def row_selected2(self, msg):
    print(msg)
    if msg.selected:
        self.row_data_div.text = msg.data
        self.row_selected2 = msg.rowIndex
    elif self.row_selected2 == msg.rowIndex:
        self.row_data_div.text = ''

def grid_test11():
    wp = jp.WebPage()
    row_data_div = jp.Div(a=wp)
    grid = wm_df.jp.ag_grid(a=wp)
    grid.row_data_div = row_data_div
    grid.on('rowSelected', row_selected2)
    grid.options.columnDefs[0].checkboxSelection = True
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("grid_test11",grid_test11)



# Justpy Tutorial demo svg_demo2 from docs/tutorial/svg_components.md
import justpy as jp
# Example based on https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill

svg_html = """
<svg viewBox="0 0 300 100" xmlns="http://www.w3.org/2000/svg" width=300 height=100>
  <!-- Simple color fill -->
  <circle cx="50" cy="50" r="40" fill="pink" />

  <!-- Fill circle with a gradient -->
  <defs>
    <radialGradient id="myGradient">
      <stop offset="0%"   stop-color="pink"/>
      <stop offset="100%" stop-color="black" />
    </radialGradient>
  </defs>

  <circle cx="150" cy="50" r="40" fill="url(#myGradient)" />

  <!--
  Keeping the final state of an animated circle
  which is a circle with a radius of 40.
  -->
  <circle cx="250" cy="50" r="20">
    <animate attributeType="XML"
             attributeName="r"
             from="0" to="40" dur="5s"
             fill="freeze" />
  </circle>
</svg>
"""


def svg_demo2():
    wp = jp.WebPage()
    jp.parse_html(svg_html, a=wp)
    return wp


# initialize the demo
from  examples.basedemo import Demo
Demo ("svg_demo2",svg_demo2)



"""
Created on 2022-08-26

@author: wf
"""
from tests.basetest import Basetest
from justpy.quasarcomponents import QInput


class TestQuasar(Basetest):
    """
    test Quasar components

    https://quasar.dev/vue-components
    """

    def testQInput(self):
        """
        test QInput
        """
        qInput = QInput()
        propList = qInput.prop_list
        debug = True
        if debug:
            print(propList)
        self.assertTrue("loading" in propList)



# Justpy Tutorial demo style_test from docs/reference/htmlcomponent.md
import justpy as jp

def style_test():
    wp = jp.WebPage()
    for size in range(1,101):
        jp.Div(text=f' {size}', style=f'font-size: {size}px; color: red', a=wp, classes='inline cursor-pointer', size=size,
               click='self.size *= 2; self.style=f"font-size: {self.size}px; color: green";')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("style_test",style_test)



"""
Created on 2022-08-25

@author: em
"""
import justpy as jp


def drag_start(self, msg):
    print("in drag start")
    print(msg)
    msg.page.image.animation = False
    return True


def drop(self, msg):
    print("in drop")
    print(msg)
    wp = msg.page
    wp.image.animation = "zoomIn"
    if self.index != wp.current_index:
        wp.div_list[self.index].add(wp.image)
        wp.div_list[wp.current_index].components = []
        wp.current_index = self.index


def drag_test():
    wp = jp.WebPage()
    wp.current_index = 0
    drag_options = {"drag_classes": "text-white bg-red-500"}
    wp.image = jp.Img(
        src="https://www.python.org/static/community_logos/python-powered-h-140x182.png",
        height=100,
        width=100,
        classes="border faster",
        drag_options=drag_options,
        animation="zoomIn",
    )
    wp.image.on("dragstart", drag_start)

    d = jp.Div(classes="flex flex-wrap", a=wp)
    wp.div_list = [
        jp.Div(
            style="height: 160px; width: 130px",
            classes="m-4 border-2 flex items-center justify-center",
            drop=drop,
            a=d,
            index=i,
        )
        for i in range(30)
    ]
    for div in wp.div_list:
        div.events.append("dragover")
    wp.div_list[0].add(wp.image)
    return wp


from examples.basedemo import Demo

Demo("drag & drop test", drag_test)



# Justpy Tutorial demo button_toggle_test from docs/quasar_tutorial/quasar_components.md
import justpy as jp

# Example from https://www.highcharts.com/docs/getting-started/your-first-chart
my_chart_def = """
{
        chart: {
            type: 'bar'
        },
        title: {
            text: 'Chart of Type Bar'
        },
        xAxis: {
            categories: ['Apples', 'Bananas', 'Oranges']
        },
        yAxis: {
            title: {
                text: 'Fruit eaten'
            }
        },
        series: [{
            name: 'Jane',
            data: [1, 0, 4]
        }, {
            name: 'John',
            data: [5, 7, 3]
        }]
}
"""

def button_change(self, msg):
    print(msg)
    self.chart.options.chart.type = self.value
    self.chart.options.title.text = f'Chart of Type {self.value}'

def button_toggle_test():
    wp = jp.QuasarPage()
    chart_type = jp.QBtnToggle(toggle_color='red', push=True, glossy=True, a=wp, input=button_change, value='bar', classes='q-ma-md')
    for type in ['bar', 'column', 'line', 'spline']:
        chart_type.options.append({'label': type.capitalize(), 'value': type})
    chart_type.chart = jp.HighCharts(a=wp, classes='q-ma-lg', options=my_chart_def)
    return wp


# initialize the demo
from  examples.basedemo import Demo
Demo ("button_toggle_test",button_toggle_test)



# Justpy Tutorial demo quasar_example4 from docs/quasar_tutorial/QExpansionItem.md
import justpy as jp

sample_text = """
Lorem ipsum dolor sit amet, consectetur adipisicing elit. Quidem, eius reprehenderit eos corrupti
            commodi magni quaerat ex numquam, dolorum officiis modi facere maiores architecto suscipit iste
            eveniet doloribus ullam aliquid.
"""

def quasar_example4():
    wp = jp.QuasarPage()
    d = jp.Div(classes="q-pa-md", style="max-width: 350px", a=wp)
    item_list = jp.QList(classes="rounded-borders", padding=True, bordered=True, a=d)
    for info in [("perm_identity", "Account settings"), ("signal_wifi_off", "Wifi settings"), ("drafts", "Drafts")]:
        expansion_item = jp.QExpansionItem(icon=info[0], label=info[1], a=item_list, header_class='text-purple',
                                           dense=True, dense_toggle=True, expand_separator=True)
        card= jp.QCard(a=expansion_item)
        jp.QCardSection(text=sample_text, a=card)

    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("quasar_example4",quasar_example4)



# Justpy Tutorial demo date_time_test from docs/quasar_tutorial/quasar_components.md
import justpy as jp

html_string = """
<div class="q-pa-md">
    <div class="q-gutter-sm">
      <q-badge color="teal" name="badge" classes="text-h3"/>
      <q-badge color="purple" text-color="white" class="q-ma-md">
        Mask: YYYY-MM-DD HH:mm
      </q-badge>
    </div>

    <div class="q-gutter-md row items-start">
      <q-date  mask="YYYY-MM-DD HH:mm" color="purple" name="date"/>
      <q-time  mask="YYYY-MM-DD HH:mm" color="purple" name="time"/>
    </div>
  </div>
"""


def date_time_test():
    wp = jp.QuasarPage(data={'date': '2020-01-01 07:00'})
    d = jp.parse_html(html_string, a=wp)
    for c in ['badge', 'date', 'time']:
        d.name_dict[c].model = [wp, 'date']
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("date_time_test",date_time_test)



# Justpy Tutorial demo custom_comp_test3 from docs/tutorial/custom_components.md
import justpy as jp

class CalendarDate(jp.Div):

    def __init__(self, **kwargs):
        self.month = 'Jan'
        self.year = '2010'
        self.weekday = 'Sun'
        self.day = '1'
        self.color = 'red'
        super().__init__(**kwargs)
        self.inner_html = f"""
        <div class="w-24 rounded-t overflow-hidden bg-white text-center m-2 cursor-default">
            <div class="bg-{self.color}-500 text-white py-1">
                {self.month}
            </div>
            <div class="pt-1 border-l border-r">
                <span class="text-4xl font-bold">{self.day}</span>
            </div>
            <div class="pb-2 px-2 border-l border-r border-b rounded-b flex justify-between">
                <span class="text-xs font-bold">{self.weekday}</span>
                <span class="text-xs font-bold">{self.year}</span>
            </div>
        </div>
        """

def custom_comp_test3():
    wp = jp.WebPage()
    year = 2019
    month = 'Feb'
    d = jp.Div(classes='flex flex-wrap', a=wp)
    for day in range(1,11):
        CalendarDate(day=day, month=month, year=year, color='teal', a=d, animation='bounceIn')
    d = jp.Div(classes='flex flex-wrap', a=wp)
    for day in range(5, 26):
        CalendarDate(day=day, month='Jul', year='2005', color='yellow', a=d)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("custom_comp_test3",custom_comp_test3)



"""
Created on 2022-09-13

@author: wf
"""
import justpy as jp
from starlette.requests import Request
from tests.base_client_test import BaseClienttest
import time
# https://fastapi.tiangolo.com/tutorial/middleware/
@jp.app.middleware("http")
async def add_process_time_header(request: Request, call_next):
    start_time = time.time()
    response = await call_next(request)
    process_time = time.time() - start_time
    response.headers["X-Process-Time"] = str(process_time)
    return response

@jp.app.route("/webpage", name="webpage")
@jp.app.response
def hello_world(_request):
    wp = jp.WebPage()
    _d = jp.Div(text='I am justpy webpage',a=wp)
    return wp

class TestMiddleWare(BaseClienttest):
    """
    test FASTAPI middleware 
    """
    
    def setUp(self, debug=False, profile=True):
        BaseClienttest.setUp(self, debug=debug, profile=profile)
        #self.app.prioritize_routes()
    
    def test_middleware(self):
        """
        test the middleware 
        """
        _response=self.checkResponse("/webpage",debug=False)
        pass
        


# Justpy Tutorial demo event_demo5 from docs/tutorial/handling_events.md
import justpy as jp

def button_click1(self, msg):
    self.num_clicked += 1
    self.message.text = f'{self.text} clicked. Number of clicks: {self.num_clicked}'
    self.set_class('bg-red-500')
    self.set_class('bg-red-700', 'hover')

def event_demo5():
    number_of_buttons = 25
    wp = jp.WebPage()
    button_div = jp.Div(classes='flex m-4 flex-wrap', a=wp)
    button_classes = 'w-32 mr-2 mb-2 bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded-full'
    message = jp.Div(text='No button clicked yet', classes='text-2xl border m-4 p-2', a=wp)
    for i in range(1, number_of_buttons + 1):
        b = jp.Button(text=f'Button {i}', a=button_div, classes=button_classes, click=button_click1)
        b.message = message
        b.num_clicked = 0
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("event_demo5",event_demo5)



# Justpy Tutorial demo plot_test1 from docs/tutorial/matplotlib.md
import justpy as jp
import matplotlib.pyplot as plt

def plot_test1():
    wp = jp.WebPage()
    plt.plot([0, 1, 4, 9], marker='*', markersize=20, markeredgecolor='red')
    plt.title('Matplotlib Example')
    plt.xlabel('x data')
    plt.ylabel('y data')
    jp.Matplotlib(a=wp)

    # If memory may be an issue, don't forget to close figures not in use anymore
    plt.close()
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("plot_test1",plot_test1)



"""
Created on 2022-10-19

see https://github.com/justpy-org/justpy/issues/571

@author: getzze
"""
#!/usr/bin/env python3
import justpy as jp
import numpy as np

button_classes = "w-32 mr-2 mb-2 bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded-full"
 
columnDefs= [
        {'headerName': 'Name', 'field': 'name'},
        {'headerName': 'Age', 'field': 'age'},
]

# The tabular data to work on (represented by a list of dicts)
lod = [
        {'name': 'Alice', 'age': 18},
        {'name': 'Bob', 'age': 21},
        {'name': 'Carol', 'age': 42},
]

def grid_test_issue571():
    """
    test issue 571
    """
    wp = jp.WebPage()
    grid = jp.AgGrid(a=wp, classes='max-h-60')
    grid.load_lod(lod=lod,columnDefs=columnDefs)

    async def update_via_rowdata(self, _msg):
        """
        update the grid content via direct rowdata access
        """
        grid.options["rowData"][1]["age"]+= 1
         
    async def update_via_load_lod(self, _msg):
        """
        update the grid content via load_lod function
        """
        lod[0]["age"] += 1
        grid.load_lod(lod=lod,columnDefs=columnDefs)
        
    async def update_with_nan(self, _msg):
        """
        update the grid content with a not a number value
        """
        lod[2]["age"] = np.nan
        grid.load_lod(lod=lod,columnDefs=columnDefs)
 
    jp.Button(text='Update via options rowData', click=update_via_rowdata, a=wp, classes=button_classes)
    jp.Button(text='Update via load_lod', click=update_via_load_lod, a=wp, classes=button_classes)
    jp.Button(text='Update with a np.nan', click=update_with_nan, a=wp, classes=button_classes)   
    return wp

from examples.basedemo import Demo
Demo("Issue 500 aggrid uuid value", grid_test_issue571)



# Justpy Tutorial demo comp_test from docs/tutorial/handling_events.md
import justpy as jp


def comp_test():
    wp = jp.WebPage()
    d = jp.Div(
        text="hello1",
        click='self.text="clicked"',
        mouseenter='self.text="entered"; self.set_class("text-5xl"); msg.page.add(Div(text=f"{len(msg.page)} Additional Div"))',
        mouseleave='self.text="left"; self.set_class("text-xl")',
        classes="text-2xl border p-2 m-2",
        a=wp,
    )
    d = jp.Div(
        text="hello2",
        click='self.text="clicked"',
        mouseenter='self.text="entered"',
        classes="text-2xl border p-2 m-2",
        a=wp,
    )
    d = jp.Div(
        text="hello3",
        click='self.text="clicked"',
        mouseenter='self.text="entered"',
        classes="text-2xl border p-2 m-2",
        a=wp,
    )
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("comp_test", comp_test)



# Justpy Tutorial demo create_chart_test3 from docs/charts_tutorial/creating_charts.md
import justpy as jp

my_chart_def = """
{
    chart: {
        type: 'bar'
    },
    xAxis: {
        categories: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
    },
    legend: {
        layout: 'vertical',
        floating: true,
        backgroundColor: '#FFFFFF',
        align: 'right',
        verticalAlign: 'top',
        y: 60,
        x: -60
    },
    tooltip: {
        formatter: function () {
            return '<b>' + this.series.name + '</b><br/>' +
                this.x + ': ' + this.y;
        }
    },
    series: [{
        data: [29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4]
    }]
}
"""


def create_chart_test3():
    wp = jp.WebPage()
    my_chart = jp.HighCharts(a=wp, classes="m-2 p-2 border w-1/2", options=my_chart_def)
    return wp


# initialize the demo
from examples.basedemo import Demo
Demo("create_chart_test3", create_chart_test3)



# Justpy Tutorial demo reactive_list_test from docs/quasar_tutorial/quasar_components.md
import justpy as jp

def check_box_clicked(self, msg):
    wp = msg.page

    def my_filter(var):
        for letter in self.value:
            if var.startswith(letter):
                return True
        return False

    filtered_list = list(filter(my_filter, wp.list_item_text))

    for c in wp.q_list.components:
        if c.components[0].text not in filtered_list:
            c.show = False
        else:
            c.show = True


def reactive_list_test():
    wp = jp.QuasarPage()
    d = jp.Div(classes="q-pa-md", style="max-width: 400px", a=wp)
    wp.list_item_text = ['apple', 'ad', 'aardvark', 'again', 'bean', 'bath', 'beauty', 'can', 'corner', 'capital']
    wp.q_list = jp.QList(dense=True, bordered=True, padding=True, classes="rounded-borders", a=d)
    for word in wp.list_item_text:
        q_item = jp.QItem(clickable=True, v_ripple=True, a=wp.q_list)
        jp.QItemSection(text=word, a=q_item)

    option_group = jp.QOptionGroup(type='checkbox', color='green', a=d, input=check_box_clicked, value=['a', 'b', 'c'],
                                   inline=True, classes='q-ma-lg',
                    options=[{'label': 'A words', 'value': 'a'}, {'label': 'B words', 'value': 'b'}, {'label': 'C words', 'value': 'c'}])

    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("reactive_list_test",reactive_list_test)



# Justpy Tutorial demo list_demo from docs/tutorial/html_components.md
import justpy as jp

def list_demo():
    wp = jp.WebPage()
    my_list = jp.Ul(a=wp, classes='m-2 p-2')
    for i in range (1,11):
        jp.Li(text=f'List one item {i}', a=my_list)
    my_list = jp.Ul(a=wp, classes='m-2 p-2 list-disc list-inside')
    for i in range(1, 11):
        jp.Li(text=f'List two item {i}', a=my_list, classes='hover:bg-gray-200')
    my_list = jp.Ul(a=wp, classes='m-2 p-2 list-decimal list-inside')
    for i in range(1, 11):
        jp.Li(text=f'List three item {i}', a=my_list)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("list_demo",list_demo)



# Justpy Tutorial demo input_test6 from docs/quasar_tutorial/QInput.md
import justpy as jp

def input_test6(request):
    wp = jp.QuasarPage()
    c1 = jp.Div(classes='q-pa-md', a=wp)
    c2 = jp.Div(classes='q-gutter-md', style='max-width: 300px', a=c1)
    icon1 = jp.QIcon(name='event', color='blue')
    icon2 = jp.QIcon(name='place', color='red')
    for slot in ['append', 'prepend', 'before']:
        in1 = jp.QInput(label=slot, filled=True, hint=f'Icon is in slot "{slot}" and "after"', a=c2, after_slot=icon2)
        #in1.after_slot = icon2    # Alternative to keyword method used in line above
        setattr(in1, slot + '_slot', icon1)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("input_test6",input_test6)



import justpy as jp

btn_classes = jp.Styles.button_outline + " m-2"
notification_classes = "m-2 text-center text-xl bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded w-64"


def btn_click(self, msg):
    self.nl[self.list_index].show = True


def btn_after(self, msg):
    self.nl[self.list_index].show = False


def after_demo():
    wp = jp.WebPage()
    btn_list = []
    notification_list = []

    for index, btn_text in enumerate(["First", "Second", "Third"]):
        btn_list.append(
            jp.Button(
                text=f"{btn_text} Button",
                classes=btn_classes,
                a=wp,
                nl=notification_list,
                list_index=index,
                click=btn_click,
                after=btn_after,
            )
        )

    for notification_text in ["First", "Second", "Third"]:
        notification_list.append(
            jp.Div(
                text=f"{notification_text} Notification",
                classes=notification_classes,
                a=wp,
                show=False,
            )
        )

    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("after demo", after_demo)



# Justpy Tutorial demo alert_test1 from docs/tutorial/custom_components.md
import justpy as jp

class MyAlert(jp.Div):

    def __init__(self, **kwargs):
        self.title_text = 'This is the title'
        self.body_text = 'This is the body'
        super().__init__(**kwargs) # Important! see below
        root = self
        c1 = jp.Div(classes='bg-teal-100 border-t-4 border-teal-500 rounded-b text-teal-900 px-4 py-3 shadow-md',
                    role='alert', a=root)
        c2 = jp.Div(classes='flex', a=c1)
        c3 = jp.Div(classes='py-1', a=c2)
        c4 = jp.Svg(classes='fill-current h-6 w-6 text-teal-500 mr-4', xmlns='http://www.w3.org/2000/svg',
                    viewBox='0 0 20 20', a=c3)
        c5 = jp.Path(
            d='M2.93 17.07A10 10 0 1 1 17.07 2.93 10 10 0 0 1 2.93 17.07zm12.73-1.41A8 8 0 1 0 4.34 4.34a8 8 0 0 0 11.32 11.32zM9 11V9h2v6H9v-4zm0-6h2v2H9V5z',
            a=c4)
        c6 = jp.Div(a=c2)
        c7 = jp.P(classes='font-bold text-lg', a=c6, text=self.title_text)
        c8 = jp.P(classes='text-sm', a=c6, text=self.body_text)

def alert_test1():
    wp = jp.WebPage()
    d = MyAlert(a=wp, classes='m-2 w-1/4', title_text='hello', body_text='How is everybody?')
    d.title_text = 'Shalom'
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("alert_test1",alert_test1)



# Justpy Tutorial demo quasar_example2 from docs/quasar_tutorial/introduction.md
import justpy as jp


html_string = """
<div class="q-pa-md" style="max-width: 350px">
    <q-list bordered>
      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-icon color="primary" name="bluetooth" />
        </q-item-section>

        <q-item-section>Icon as avatar</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar color="teal" text-color="white" icon="bluetooth" />
        </q-item-section>

        <q-item-section>Avatar-type icon</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar rounded color="purple" text-color="white" icon="bluetooth" />
        </q-item-section>

        <q-item-section>Rounded avatar-type icon</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar color="primary" text-color="white">
            R
          </q-avatar>
        </q-item-section>

        <q-item-section>Letter avatar-type</q-item-section>
      </q-item>

      <q-separator />

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar>
            <img src="https://cdn.quasar.dev/img/boy-avatar.png">
          </q-avatar>
        </q-item-section>
        <q-item-section>Image avatar</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar square>
            <img src="https://cdn.quasar.dev/img/boy-avatar.png">
          </q-avatar>
        </q-item-section>
        <q-item-section>Image square avatar</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar rounded>
            <img src="https://cdn.quasar.dev/img/boy-avatar.png">
          </q-avatar>
        </q-item-section>
        <q-item-section>Image rounded avatar</q-item-section>
      </q-item>

      <q-separator />

      <q-item clickable v-ripple>
        <q-item-section avatar>
          <q-avatar rounded>
            <img src="https://cdn.quasar.dev/img/mountains.jpg">
          </q-avatar>
        </q-item-section>
        <q-item-section>List item</q-item-section>
      </q-item>

      <q-item clickable v-ripple>
        <q-item-section thumbnail>
          <img src="https://cdn.quasar.dev/img/mountains.jpg">
        </q-item-section>
        <q-item-section>List item</q-item-section>
      </q-item>
    </q-list>
  </div>
"""

def quasar_example2():
    wp = jp.QuasarPage()
    c = jp.parse_html(html_string, a=wp)
    for i in c.commands:
        print(i)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("quasar_example2",quasar_example2)



# Justpy Tutorial demo plot_test9 from docs/tutorial/matplotlib.md
import justpy as jp
import seaborn as sns

# All examples from https://seaborn.pydata.org/introduction.html

def create_page():
    wp = jp.WebPage(delete_flag=False)
    chart_classes = 'm-2'

    jp.Div(text='Tips Dataset', a=wp, classes='text-white bg-blue-500 text-center text-xl')
    sns.set()
    tips = sns.load_dataset("tips")
    sns.relplot(x="total_bill", y="tip", col="time",
                hue="smoker", style="smoker", size="size",
                data=tips)
    jp.Matplotlib(a=wp, classes=chart_classes)
    sns.lmplot(x="total_bill", y="tip", col="time", hue="smoker",
               data=tips)
    jp.Matplotlib(a=wp, classes=chart_classes)
    sns.catplot(x="day", y="total_bill", hue="smoker",
                kind="swarm", data=tips)
    jp.Matplotlib(a=wp, classes=chart_classes)
    sns.catplot(x="day", y="total_bill", hue="smoker",
                kind="violin", split=True, data=tips)
    jp.Matplotlib(a=wp, classes=chart_classes)

    jp.Div(text='Dots Dataset', a=wp, classes='text-white bg-blue-500 text-center text-xl')
    dots = sns.load_dataset("dots")
    sns.relplot(x="time", y="firing_rate", col="align",
                hue="choice", size="coherence", style="choice",
                facet_kws=dict(sharex=False),
                kind="line", legend="full", data=dots)
    jp.Matplotlib(a=wp, classes=chart_classes)

    jp.Div(text='FMRI Dataset', a=wp, classes='text-white bg-blue-500 text-center text-xl')
    fmri = sns.load_dataset("fmri")
    sns.relplot(x="timepoint", y="signal", col="region",
                hue="event", style="event",
                kind="line", data=fmri)
    jp.Matplotlib(a=wp, classes=chart_classes)

    jp.Div(text='Iris Dataset', a=wp, classes='text-white bg-blue-500 text-center text-xl')
    iris = sns.load_dataset("iris")
    sns.jointplot(x="sepal_length", y="petal_length", data=iris)
    jp.Matplotlib(a=wp, classes=chart_classes)
    sns.pairplot(data=iris, hue="species")
    jp.Matplotlib(a=wp, classes=chart_classes)

    return wp


wp = create_page()

def plot_test9():
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("plot_test9",plot_test9)



# Justpy Tutorial demo link_demo2 from docs/tutorial/html_components.md
import justpy as jp

def link_demo2():
    wp = jp.WebPage()
    link = jp.A(text='Scroll to target', a=wp, classes='inline-block m-2 p-2 text-xl text-white bg-blue-500 hover:bg-blue-700')
    # jp.Br(a=wp)
    for i in range(50):
        jp.P(text=f'{i+1} Not a target', classes='m-1 p-1 text-white bg-blue-300', a=wp)
    target = jp.A(text=f'This is the target - it is linked to first link, click to jump there', classes='inline-block m-1 p-1 text-white bg-red-500', a=wp)
    link.bookmark = target
    link.scroll = True
    target.bookmark = link
    for i in range(50):
        jp.P(text=f'{i+50} Not a target', classes='m-1 p-1 text-white bg-blue-300', a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("link_demo2",link_demo2)



# Justpy Tutorial demo quasar_example3 from docs/quasar_tutorial/introduction.md
import justpy as jp

def quasar_example3():
    wp = jp.QuasarPage()
    d = jp.Div(classes='q-pa-md row justify-center', a=wp)
    jp.QDiv(v_ripple=True, classes='relative-position flex flex-center text-white bg-primary',
                style='border-radius: 3px; cursor: pointer; height: 150px; width: 80%;',
                a=d, text='Click/tap me')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("quasar_example3",quasar_example3)



# Justpy Tutorial demo input_test7 from docs/quasar_tutorial/QInput.md
import justpy as jp

def input_test7(request):
    wp = jp.QuasarPage()
    c1 = jp.Div(classes='q-pa-md', a=wp)
    c2 = jp.Div(classes='q-gutter-md', style='max-width: 300px', a=c1)

    password_input = jp.QInput(filled=True,  type='password', a=c2, hint="Password with toggle")
    visibility_icon = jp.QIcon(name='visibility_off', classes='cursor-pointer')
    visibility_icon.password_input = password_input

    password_input.append_slot = visibility_icon

    def toggle_password(self, msg):
        if self.name == 'visibility_off':
            self.name = 'visibility'
            self.password_input.type='text'
        else:
            self.name = 'visibility_off'
            self.password_input.type = 'password'

    visibility_icon.on('click', toggle_password)

    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("input_test7",input_test7)



# Justpy Tutorial demo color_demo from docs/tutorial/input.md
import justpy as jp

def color_demo(request):
    wp = jp.WebPage()
    in1 = jp.Input(type='color', a=wp, classes='m-2 p-2', style='width: 100px; height: 100px', input=color_change, debounce=30)
    in1.d = jp.Div(text='Click box above to change color of this text', a=wp, classes='border m-2 p-2 text-2xl font-bold')
    return wp

def color_change(self, msg):
    self.d.style=f'color: {self.value}'
    self.d.text = f'The color of this text is: {self.value}'

# initialize the demo
from  examples.basedemo import Demo
Demo ("color_demo",color_demo)



# Justpy Tutorial demo event_demo2 from docs/tutorial/handling_events.md
import justpy as jp

def my_click2(self, msg):
    self.text = 'I was clicked'
    print(msg.event_type)
    print(msg['event_type'])
    print(msg)

def event_demo2():
    wp = jp.WebPage()
    d = jp.P(text='Not clicked yet', a=wp, classes='text-xl m-2 p-2 bg-blue-500 text-white')
    d.on('click', my_click2)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("event_demo2",event_demo2)



"""
Created on 2022-08-31
@author: wf
"""
from tests.basetest import Basetest
from mimetypes import guess_type
from starlette.responses import FileResponse


class TestMimetypes(Basetest):
    """
    test the mimetype handling
    """

    def testMimeTypes(self):
        """
        test the mimetypes
        """
        examples = [
            (
                "js",
                ["text/javascript", "application/javascript"],
            ),  # https://stackoverflow.com/questions/21098865/text-javascript-vs-application-javascript
            ("html", ["text/html"]),
            ("css", ["text/css"]),
            ("svg", ["image/svg+xml"]),
        ]
        for ext, expected in examples:
            filename = f"somefile.{ext}"
            mtype, _strictness = guess_type(filename)
            smtype = FileResponse(filename).media_type
            print(f"{ext}:{mtype}:{smtype}")
            self.assertIn(mtype, expected)
            self.assertIn(smtype, expected)



"""
Created on 2022-09-17

@author: th
"""
import asyncio
import unittest

from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC

import justpy as jp
from tests.base_selenium_test import BaseSeleniumTest



class TestIssue38(BaseSeleniumTest):
    """
    testing issue 38
    https://github.com/justpy-org/justpy/issues/38
    """

    async def setUp(self):
        await BaseSeleniumTest.setUp(self,port=8124)

    async def tearDown(self):
        self.browser.close()
        await asyncio.sleep(self.server.sleep_time)
        await self.server.stop()

    def my_input3(self, msg):
        value = self.in1.value
        print(value)
        self.div.text = self.in1.value

    def input_demo4(self, request):
        input_classes = "m-2 bg-gray-200 border-2 border-gray-200 rounded w-64 py-2 px-4 text-gray-700 " \
                        "focus:outline-none focus:bg-white focus:border-purple-500 "
        p_classes = 'm-2 p-2 h-32 text-xl border-2'
        wp = jp.WebPage()
        self.in1 = jp.Input(name="inputField", type='number', a=wp, classes=input_classes, placeholder='Please type here')
        self.div = jp.Div(name="displayDiv", text='What you type will show up here', classes=p_classes, a=wp)
        self.in1.on('change', self.my_input3)
        return wp

    async def test_issue_38(self):
        """
        test when the issue occurs that input and input display differ
        """
        await self.server.start(self.input_demo4)
        url = self.server.get_url("/")
        self.browser.get(url)
        await asyncio.sleep(self.server.sleep_time)
        display_div_locator = (By.NAME, "displayDiv")
        input_field = self.browser.find_elements(By.NAME, "inputField")[0]
        display_div = self.browser.find_elements(*display_div_locator)[0]
        wait_browser= self.get_waiting_browser(self.browser, 2)
        value = ""
        # For the numbers of length 16 the issue does not occur
        # after adding the 17 char the input field starts to diverge
        for i in range(1,18):
            value += "1"
            input_field.send_keys("1")
            display_div.click()  # lose focus
            self.assertTrue(
                    wait_browser.until(
                            EC.text_to_be_present_in_element(
                                    locator=display_div_locator,
                                    text_=value
                            )),
                    f"After {i} numbers the issue occurs (unexpected)")
        # now value and the input_field are not the same → chang input_field to verify
        value += "1"
        input_field.send_keys("1")
        display_div.click()  # lose focus
        self.assertNotEqual(value, display_div.text)


# see https://github.com/justpy-org/justpy/issues/279
import justpy as jp
import time


def button_click(self, _msg):
    """
    on button click delete the components of the button
    """
    self.num_clicked += 1
    self.message.text = f"{self.text} clicked. Number of clicks: {self.num_clicked}"
    self.set_class("bg-red-500")
    self.set_class("bg-red-700", "hover")
    self.button_div.delete_components()
    # wait a bit so that another button may be clicked
    time.sleep(3)


def issue_279():
    """
    show misbehavior of handle_events when a message for a non existing component arrives
    """
    number_of_buttons = 25
    wp = jp.WebPage()
    button_div = jp.Div(classes="flex m-4 flex-wrap", a=wp)
    button_classes = "w-32 mr-2 mb-2 bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded-full"
    message = jp.Div(
        text="No button clicked yet", classes="text-2xl border m-4 p-2", a=wp
    )
    for i in range(1, number_of_buttons + 1):
        b = jp.Button(
            text=f"Button {i}",
            a=button_div,
            classes=button_classes,
            click=button_click,
            button_div=button_div,
        )
        b.message = message
        b.num_clicked = 0
    return wp


from examples.basedemo import Demo

Demo("Issue 279 KeyError", issue_279)



# Justpy Tutorial demo reload_demo from docs/tutorial/ajax.md
import justpy as jp
import asyncio

wp = jp.WebPage(delete_flag=False)
wp.reload_interval = 2.5
count_div = jp.Div(a=wp, classes='text-center m-4 p-4 text-white bg-blue-500', style='font-size: 200px')

async def increment_counter(start):
    count_div.counter = start
    while True:
        count_div.counter += 1
        count_div.text = str(count_div.counter)
        await asyncio.sleep(1)

def start_counting():
    jp.run_task(increment_counter(500))

def reload_demo():
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("reload_demo",reload_demo, startup=start_counting, websockets=False)



# Justpy Tutorial demo svg_demo1 from docs/tutorial/svg_components.md
import justpy as jp

def svg_demo1():
    wp = jp.WebPage()
    for color in ['red', 'green', 'blue']:
        svg = jp.Svg(viewBox='0 0 100 100', xmlns='http://www.w3.org/2000/svg', a=wp, width=100, height=100, classes='m-2 inline-block')
        circle = jp.Circle(cx='50', cy='50', r='50', fill=color, a=svg)
    for radius in range(10, 51, 10):
        svg = jp.Svg(viewBox='0 0 100 100', xmlns='http://www.w3.org/2000/svg', a=wp, width=100, height=100, classes='m-2 inline-block')
        ellipse = jp.Ellipse(cx=50, cy=50, rx=radius, ry=radius/2, fill='teal', a=svg)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("svg_demo1",svg_demo1)



"""
Created on 2022-08-25

@author: wf
"""
import asyncio
import selenium.common
from selenium.webdriver.common.by import By
from testfixtures import LogCapture
from tests.base_selenium_test import BaseSeleniumTest

class TestIssue279WithSelenium(BaseSeleniumTest):
    """
    testing actual browser behavior with selenium
    """

    async def setUp(self):
        await super().setUp(port=8125)
   
    async def testIssue279(self):
        """
        see https://github.com/justpy-org/justpy/issues/279

        """
        self.browser = await self.getBrowserForDemo()
        from examples.issues.issue_279_key_error import issue_279

        await self.server.start(issue_279)
        url = self.server.get_url("/")
        self.browser.get(url)
        await asyncio.sleep(self.server.sleep_time)
        buttons = self.browser.find_elements(By.TAG_NAME, "button")
        debug = True
        if debug:
            print(f"found {len(buttons)} buttons")
        await asyncio.sleep(0.5)
        ok = "False"
        with LogCapture() as lc:
            try:
                for buttonIndex in [0, 1, 2, 3]:
                    buttons[buttonIndex].click()
                    await asyncio.sleep(0.25)
                await asyncio.sleep(1.0)
                for buttonIndex in [0, 1, 2, 3]:
                    buttons[buttonIndex].click()
                    await asyncio.sleep(0.25)
            except selenium.common.exceptions.StaleElementReferenceException:
                if debug:
                    print(
                        "Expected sideeffect: Selenium already complains about missing button"
                    )
                ok = True
                pass
            await asyncio.sleep(3.2)
            if debug:
                print(f"log capture: {str(lc)}")
            expecteds = [
                "component with id",
                "doesn't exist (anymore ...) it might have been deleted before the event handling was triggered",
            ]
            for i, expected in enumerate(expecteds):
                if not ok:
                    self.assertTrue(expected in str(lc), f"{i}:{expected}")
                else:
                    if not expected in str(lc):
                        print(f"{i}:{expected} missing in captured log")

        self.browser.close()
        await self.server.stop()


# Justpy Tutorial demo custom_comp_test2 from docs/tutorial/custom_components.md
import justpy as jp

class PillButton(jp.Button):

    def __init__(self, **kwargs):
        self.bg_color = 'blue'
        super().__init__(**kwargs)
        self.set_classes(f'bg-{self.bg_color}-500 hover:bg-{self.bg_color}-700 text-white font-bold py-2 px-4 rounded-full')

def custom_comp_test2():
    wp = jp.WebPage()
    for color in ['blue', 'red', 'orange', 'pink']:
        PillButton(bg_color=color, text='Pill Button', click='self.text="I was clicked"', a=wp, classes='m-2')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("custom_comp_test2",custom_comp_test2)



# Justpy Tutorial demo alpine_test from docs/blog/vue_comparison.md
import justpy as jp

class SimpleDropdown(jp.Div):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        b = jp.Button(text='Open Dropdown', a=self, click='self.u.show = not self.u.show')
        b.u = jp.Ul(a=self, text='Drop Down Body', show=False, click__out='self.show = False')

def alpine_test():
    wp =jp.WebPage()
    SimpleDropdown(a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("alpine_test",alpine_test)



# Justpy Tutorial demo result_test from docs/tutorial/page_events.md
import justpy as jp

js_string = """
a = 3;
b = 5;
c = a * b;
d = {r: c, appName: navigator.appName, appVersion: navigator.appVersion}

"""


async def page_ready(self, msg):
    jp.run_task(self.run_javascript(js_string))


async def result_ready(self, msg):
    msg.page.result_div.text = f"The result is: {msg.result}"


def result_test():
    wp = jp.WebPage()
    wp.on("page_ready", page_ready)
    wp.on("result_ready", result_ready)
    wp.result_div = jp.Div(text="Result will go here", classes="m-4 p-2 text-xl", a=wp)
    jp.Pre(text=js_string, a=wp, classes="m-2 p-2 border")
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("result_test", result_test)



# Justpy Tutorial demo entity_test from docs/reference/htmlcomponent.md
import justpy as jp

def entity_test():
    wp = jp.WebPage()
    jp.Space(num=3, a=wp)
    jp.HTMLEntity(entity='a&#768;',a=wp, classes='text-lg')
    jp.Span(text='a&#768;',a=wp, classes='text-lg', html_entity=True)
    jp.Space(num=5, a=wp)
    jp.HTMLEntity(entity='a&#769',a=wp, classes='text-xl')
    jp.Span(text='a&#769',a=wp, classes='text-xl', html_entity=True)
    jp.Space(num=5, a=wp)
    jp.HTMLEntity(entity='&#8707;',a=wp, classes='text-2xl')
    jp.Span(text='&#8707;',a=wp, classes='text-2xl', html_entity=True)
    jp.Space(num=5, a=wp)
    jp.HTMLEntity(entity='&copy;', a=wp, classes='text-3xl')
    jp.Span(text='&copy;', a=wp, classes='text-3xl', html_entity=True)
    jp.Space(num=5, a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("entity_test",entity_test)



# Justpy Tutorial demo tab_comp_test1 from docs/tutorial/custom_components.md
from justpy import Div, WebPage, Ul, Li, HighCharts, A, justpy
import justpy as jp


class Tabs(Div):

    tab_label_classes = 'overflow-hidden cursor-pointer bg-white inline-block py-2 px-4 text-blue-500 hover:text-blue-800 font-semibold'
    tab_label_classes_selected = 'overflow-hidden cursor-pointer bg-white inline-block border-l border-t border-r rounded-t py-2 px-4 text-blue-700 font-semibold'
    item_classes = 'flex-shrink mr-1'
    item_classes_selected = 'flex-shrink -mb-px mr-1'
    wrapper_style = 'display: flex; position: absolute; width: 100%; height: 100%;  align-items: center; justify-content: center; background-color: #fff;'

    def __init__(self, **kwargs):

        self.tabs = []  # list of {'id': id, 'label': label, 'content': content}
        self.value = None  # The value of the tabs component is the id of the selected tab
        self.content_height = 500
        self.last_rendered_value = None
        self.animation = False
        self.animation_next = 'slideInRight'
        self.animation_prev = 'slideOutLeft'
        self.animation_speed = 'faster'  # '' | 'slow' | 'slower' | 'fast'  | 'faster'

        super().__init__(**kwargs)

        self.tab_list = Ul(classes="flex flex-wrap border-b", a=self)
        self.content_div = Div(a=self)
        self.delete_list = []


    def __setattr__(self, key, value):
        if key == 'value':
            try:
                self.previous_value = self.value
            except:
                pass
        self.__dict__[key] = value

    def add_tab(self, id, label, content):
        self.tabs.append({'id': id, 'label': label, 'content': content})
        if not self.value:
            self.value = id

    def get_tab_by_id(self, id):
        for tab in self.tabs:
            if tab['id'] == id:
                return tab
        return None

    def set_content_div(self, tab):
        self.content_div.add(tab['content'])
        self.content_div.set_classes('relative overflow-hidden border')
        self.content_div.style = f'height: {self.content_height}px'

    def set_content_animate(self, tab):
        self.wrapper_div_classes = self.animation_speed  # Component in this will be centered

        if self.previous_value:
            self.wrapper_div = Div(classes=self.wrapper_div_classes, animation=self.animation_next, temp=True,
                                   style=f'{self.wrapper_style} z-index: 50;', a=self.content_div)
            self.wrapper_div.add(tab['content'])
            self.wrapper_div = Div(classes=self.wrapper_div_classes, animation=self.animation_prev, temp=True,
                                   style=f'{self.wrapper_style} z-index: 0;', a=self.content_div)
            self.wrapper_div.add(self.get_tab_by_id(self.previous_value)['content'])
        else:
            self.wrapper_div = Div(classes=self.wrapper_div_classes, temp=True, a=self.content_div,
                                   style=self.wrapper_style)
            self.wrapper_div.add(tab['content'])

        self.content_div.set_classes('relative overflow-hidden border')
        self.content_div.style = f'height: {self.content_height}px'


    def model_update(self):
        val = self.model[0].data[self.model[1]]
        if self.get_tab_by_id(val):
            self.value = val

    def delete(self):
        for c in self.delete_list:
            c.delete_flag = True
            c.delete()
            c.needs_deletion = False

        if self.delete_flag:
            for tab in self.tabs:
                tab['content'].delete()
                tab['content'] = None
        super().delete()

    @staticmethod
    async def tab_click(self, msg):
        if self.tabs.value != self.tab_id:
            previous_tab = self.tabs.value
            self.tabs.value = self.tab_id
            if hasattr(self.tabs, 'model'):
                self.tabs.model[0].data[self.tabs.model[1]] = self.tabs.value
            # Run change if it exists
            if self.tabs.has_event_function('change'):
                msg.previous_tab = previous_tab
                msg.new_tab = self.tabs.value
                msg.id = self.tabs.id
                msg.value = self.tabs.value
                msg.class_name = self.tabs.__class__.__name__
                return await self.tabs.run_event_function('change', msg)
        else:
            return True  # No need to update page

    def convert_object_to_dict(self):
        if hasattr(self, 'model'):
            self.model_update()
        self.set_classes('flex flex-col')
        self.tab_list.delete_components()
        self.content_div.components = []
        for tab in self.tabs:
            if tab['id'] != self.value:
                tab_li = Li(a=self.tab_list, classes=self.item_classes)
                li_item = A(text=tab['label'], classes=self.tab_label_classes, a=tab_li, delete_flag=False)
                self.delete_list.append(li_item)
            else:
                tab_li = Li(a=self.tab_list, classes=self.item_classes_selected)
                li_item = A(text=tab['label'], classes=self.tab_label_classes_selected, a=tab_li, delete_flag=False)
                self.delete_list.append(li_item)
                if self.animation and (self.value != self.last_rendered_value):
                    self.set_content_animate(tab)
                else:
                    self.set_content_div(tab)
            li_item.tab_id = tab['id']
            li_item.tabs = self
            li_item.on('click', self.tab_click)
        self.last_rendered_value = self.value
        d = super().convert_object_to_dict()

        return d


class TabsPills(Tabs):
    tab_label_classes = 'cursor-pointer inline-block border border-white rounded hover:border-gray-200 text-blue-500 hover:bg-gray-200 py-1 px-3'
    tab_label_classes_selected = 'cursor-pointer inline-block border border-blue-500 rounded py-1 px-3 bg-blue-500 text-white'
    item_classes = 'flex-shrink mr-3'
    item_classes_selected = 'flex-shrink -mb-px mr-3'


my_chart_def = """
{
        chart: {
            type: 'bar'
        },
        title: {
            text: 'Fruit Consumption'
        },
        xAxis: {
            categories: ['Apples', 'Bananas', 'Oranges']
        },
        yAxis: {
            title: {
                text: 'Fruit eaten'
            }
        },
        series: [{
            name: 'Jane',
            data: [1, 0, 4],
            animation: false
        }, {
            name: 'John',
            data: [5, 7, 3],
            animation: false
        }]
}
"""
# https://dog.ceo/api/breed/papillon/images/random
pics_french_bulldogs = ['5458', '7806', '5667', '4860']
pics_papillons = ['5037', '2556', '7606', '8241']

def tab_change(self, msg):
    print('in change', msg)

def tab_comp_test1():
    wp = jp.WebPage(data={'tab': 'id2556'})

    t = Tabs(a=wp, classes='w-3/4 m-4', style='', animation=True, content_height=550)
    for chart_type in ['bar', 'column', 'line', 'spline']:
        d = jp.Div(style=Tabs.wrapper_style, delete_flag=True)
        my_chart = jp.HighCharts(a=d, classes='m-2 p-2 border', style='width: 1000px;', options=my_chart_def, use_cache=False)
        my_chart.options.chart.type = chart_type
        my_chart.options.title.text = f'Chart of Type {chart_type.capitalize()}'
        my_chart.options.subtitle.text = f'Subtitle {chart_type.capitalize()}'
        t.add_tab(f'id{chart_type}', f'{chart_type}', d)

    d_flex = Div(classes='flex', a=wp)  # Container for the two dog pictures tabs

    t = Tabs(a=d_flex, classes=' w-1/2 m-4', animation=True, content_height=550, model=[wp, 'tab'], change=tab_change)
    for pic_id in pics_papillons:
        d = jp.Div(style=Tabs.wrapper_style)
        jp.Img(src=f'https://images.dog.ceo/breeds/papillon/n02086910_{pic_id}.jpg', a=d)
        t.add_tab(f'id{pic_id}', f'Pic {pic_id}', d)

    t = TabsPills(a=d_flex, classes='w-1/2 m-4', animation=True, content_height=550, change=tab_change)
    for pic_id in pics_french_bulldogs:
        d = jp.Div(style=Tabs.wrapper_style)
        jp.Img(src=f'https://images.dog.ceo/breeds/bulldog-french/n02108915_{pic_id}.jpg', a=d)
        t.add_tab(f'id{pic_id}', f'Pic {pic_id}', d)

    input_classes = "w-1/3 m-2 bg-gray-200 border-2 border-gray-200 rounded w-64 py-2 px-4 text-gray-700 focus:outline-none focus:bg-white focus:border-purple-500"

    in1 = jp.Input(classes=input_classes, model=[wp, 'tab'], a=wp)

    return wp


# initialize the demo
from  examples.basedemo import Demo
Demo ("tab_comp_test1",tab_comp_test1)



# Justpy Tutorial demo chart_test3 from docs/charts_tutorial/tooltips.md
import justpy as jp

my_chart_def = """
{
    chart: {
        type: 'column'
    },
    title: {
        text: 'Tourists in Middle Earth'
        },
    xAxis: {
        categories: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
    },
    series: [{
        name: "Tourists in '000",
        data: [29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4]
    }
    ]
}
"""

async def tooltip_formatter5(self, msg):
    tooltip_html = f"""
    <div style="color: {msg.color};">{msg.series_name}</div>
    <div>{msg.category}</div>
    <div>x: {msg.x} y: {msg.y}</div>
    """
    return await self.tooltip_update(tooltip_html, msg.websocket)

def chart_test3():
    wp = jp.WebPage()
    # First chart
    chart1 = jp.HighCharts(a=wp, classes='m-2 p-2 border w-1/2', options=my_chart_def)
    chart1.options.subtitle.text = 'Default Tooltip'

    # Second chart
    chart2 = jp.HighCharts(a=wp, classes='m-2 p-2 border w-1/2', options=my_chart_def)
    chart2.options.subtitle.text = 'Custom Tooltip'
    chart2.on('tooltip', tooltip_formatter5)
    chart2.tooltip_fixed = True
    chart2.tooltip_y = 100
    chart2.tooltip_x = 100
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("chart_test3",chart_test3)



# Justpy Tutorial demo for_example4 from docs/blog/vue_comparison.md
import justpy as jp


def for_example4():
    wp = jp.WebPage(tailwind=False)
    object = {
      'title': 'How to do lists in Vue',
      'author': 'Jane Doe',
      'publishedAt': '2016-04-10'
    }
    for index, (name, value) in enumerate(object.items()):
        jp.Div(text=f'{index}. {name}: {value}', a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("for_example4",for_example4)



# Justpy Tutorial demo grid_test13 from docs/grids_tutorial/grid_events.md
import justpy as jp

grid_options = """
{
    rowSelection: 'multiple',
    defaultColDef: {
        filter: true,
        sortable: true,
        resizable: true,
        cellStyle: {textAlign: 'center'},
        headerClass: 'font-bold'
    }, 
      columnDefs: [
      {headerName: "Make", field: "make"},
      {headerName: "Model", field: "model"},
      {headerName: "Price", field: "price"}
    ],
      rowData: [
      {make: "Toyota", model: "Celica", price: 35000},
      {make: "Ford", model: "Mondeo", price: 32000},
      {make: "Porsche", model: "Boxter", price: 72000}
    ]
}
"""

def row_selected4(self, msg):
    print(msg.selected, msg)
    wp = msg.page
    if msg.selected:
        wp.selected_rows[msg.rowIndex] = msg.data
    else:
        wp.selected_rows.pop(msg.rowIndex)
    s = f'Selected rows {sorted(list(wp.selected_rows.keys()))}'
    for i in sorted(wp.selected_rows):
        s = f'{s}\n Row {i}  Data: {wp.selected_rows[i]}'
    if wp.selected_rows:
        wp.rows_div.text = s
    else:
        wp.rows_div.text = 'No row selected'


def grid_test13():
    wp = jp.WebPage()
    wp.selected_rows = {}  # Dictionary holding selected rows
    grid = jp.AgGrid(a=wp, options=grid_options, style='height: 200px; width: 300px; margin: 0.25em')
    grid.options.columnDefs[0].checkboxSelection = True
    grid.on('rowSelected', row_selected4)
    wp.rows_div = jp.Pre(text='Data will go here when you select rows', classes='border text-lg', a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("grid_test13",grid_test13)



# Justpy Tutorial demo hello_test2 from docs/tutorial/custom_components.md
import justpy as jp

def hello_test2():
    wp = jp.WebPage()
    for i in range(5):
        wp.add(jp.Hello()) # or jp.Hello(a=wp)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("hello_test2",hello_test2)



'''
Created on 17.09.2022

@author: wf
'''
import re

class ExampleSource:
    """
    source of an example - holding details such as the logo and url
    """
    
    def __init__(self,source_type:str="tutorial",description:str="",url:str=None,icon_size:int=32):
        """
        construct me with the given type and set the
        logo of the source type
        
        Args:
            source_type(str): e.g. tutorial/stackoverflow/issue
            url(str): the url
            icon_size(int): the size of the logo icon of the source_type, default: 32
        """
        self.source_type=source_type
        if source_type=="stackoverflow":
            # https://stackoverflow.design/brand/logo/
            self.logo_url="https://upload.wikimedia.org/wikipedia/commons/thumb/e/ef/Stack_Overflow_icon.svg/32px-Stack_Overflow_icon.svg.png"
            pass
        elif source_type=="github issue":
            # https://github.com/logos
            self.logo_url="https://upload.wikimedia.org/wikipedia/commons/thumb/9/91/Octicons-mark-github.svg/32px-Octicons-mark-github.svg.png"
            pass
        else:
            # 32x32 pixel logo
            self.logo_url="https://elimintz.github.io/favicon.png"
            pass
        self.url=url
        self.description=description
        self.icon_size=icon_size
        
    @property
    def img_link(self):
        img_link=f"""<a href="{self.url}" target="_blank"><img src="{self.logo_url}" alt="{self.description}" title="{self.description}" style="width:{self.icon_size}px;height:{self.icon_size}px;"></a>"""
        return img_link
    
    def read_source(self,path:str):
        """
        read the source from the given path
        
        Args:
            path(str): the path to the module file
        """
        with open(path, "r") as source_file:
            self.source = source_file.read()
        self.lines=self.source.split("\n")
        
    @classmethod
    def of_path(cls,path:str):
        """
        derive a example source type from the given path
        
        Args:
            path(str): the path to the module file
            
        """
        issue_match=re.search("issue_([0-9]+)",path)
        if issue_match:
            issue_number=issue_match.group(1)
            issue_url=f"https://github.com/justpy-org/justpy/issues/{issue_number}"
            example_source=ExampleSource("github issue",description=f"github issue {issue_number}",url=issue_url)
        else:    
            stackoverflow_match=re.search("stackoverflow[/]q([0-9]+)",path)
            if stackoverflow_match:
                stackoverflow_number=stackoverflow_match.group(1)
                stackoverflow_url=f"https://stackoverflow.com/questions/{stackoverflow_number}"
                example_source=ExampleSource("stackoverflow",description=f"stackoverflow question {stackoverflow_number}",url=stackoverflow_url)
            else:
                example_source=ExampleSource()
        example_source.read_source(path)
        return example_source
    
    def __str__(self):
        """
        return my string representation
        
        Returns:
            str: a text representation
        """
        text=f"{self.source_type}:{self.description}({len(self.lines)} lines)" 
        return text
        
class Example:
    """
    an example
    """
    
    def __init__(self,example_source:ExampleSource,name:str,option:str=None,header:str="",lines:list=[]):
        """
        constructor
        
        Args:
            example_source(example_source): the source of this example e.g. tutorial/issue/stackoverflow Question
            name(str): the name of the example
            option(str): option
            header(str): the header/description of the example
            lines(list): the source code lines
        """
        self.example_source=example_source
        self.name=name
        self.option=option
        self.header=header
        self.example_source.lines=lines
        
    def __str__(self):
        """
        return my string representation
        
        Returns:
            str: a text representation
        """
        text=f"{self.name}"
        return text


# Justpy Tutorial demo hello_world3 from docs/tutorial/basic_concepts.md
import justpy as jp


def hello_world3():
    wp = jp.WebPage()
    my_paragraph_design = (
        "w-64 bg-blue-500 m-2 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded"
    )
    for i in range(1, 11):
        jp.P(text=f"{i}) Hello World!", a=wp, classes=my_paragraph_design)
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("hello_world3", hello_world3)



# Justpy Tutorial demo create_chart_test2 from docs/charts_tutorial/creating_charts.md
import justpy as jp

# Example from https://www.highcharts.com/docs/getting-started/your-first-chart
my_chart_def = """
{
        chart: {
            type: 'bar'
        },
        title: {
            text: 'Fruit Consumption'
        },
        xAxis: {
            categories: ['Apples', 'Bananas', 'Oranges']
        },
        yAxis: {
            title: {
                text: 'Fruit eaten'
            }
        },
        series: [{
            name: 'Jane',
            data: [1, 0, 4]
        }, {
            name: 'John',
            data: [5, 7, 3]
        }]
}
"""


def create_chart_test2():
    wp = jp.WebPage()
    for chart_type in ["bar", "column", "line", "spline"]:
        my_chart = jp.HighCharts(a=wp, classes="m-2 p-2 border w-1/2", options=my_chart_def)
        my_chart.options.chart.type = chart_type
        my_chart.options.title.text = f"Chart of Type {chart_type.capitalize()}"
        my_chart.options.subtitle.text = f"Subtitle {chart_type.capitalize()}"
    return wp

# initialize the demo
from examples.basedemo import Demo
Demo("create_chart_test2", create_chart_test2)



# Justpy Tutorial demo joint_edit from docs/tutorial/pushing_data.md
import justpy as jp

wp = jp.QuasarPage(delete_flag=False)
editor = jp.QEditor(a=wp, kitchen_sink=True, height='94vh')

def joint_edit():
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("joint_edit",joint_edit)



# Justpy Tutorial demo stock_test2 
# generated by demoapp.update_from tutorial at 2022-09-21T09:56:14.196438+00:00 
# see https://github.com/justpy-org/justpy/blob/master/examples/charts_tutorial/stock_charts/stock_test2.py
# 
# Stock Chart with Volume            
# see https://justpy.io/charts_tutorial/stock_charts#stock-chart-with-volume
import justpy as jp
import pandas as pd
import datetime

epoch = datetime.datetime(1970, 1, 1)
grouping_units = [["week", [1]], ["month", [1, 2, 3, 4, 6]]]

chart_dict = {
    "rangeSelector": {"selected": 1},
    "yAxis": [
        {
            "labels": {"align": "right", "x": -3},
            "title": {"text": "OHLC"},
            "height": "60%",
            "lineWidth": 2,
            "resize": {"enabled": True},
        },
        {
            "labels": {"align": "right", "x": -3},
            "title": {"text": "Volume"},
            "top": "65%",
            "height": "35%",
            "offset": 0,
            "lineWidth": 2,
        },
    ],
    "tooltip": {"split": True},
    "series": [
        {
            "type": "candlestick",
            "tooltip": {"valueDecimals": 2},
            "dataGrouping": {"units": grouping_units},
        },
        {
            "type": "column",
            "name": "Volume",
            "yAxis": 1,
            "dataGrouping": {"units": grouping_units},
        },
    ],
}


def convert_date2(date_string):
    date = datetime.datetime.strptime(date_string, "%Y-%m-%d")
    return (date - epoch).total_seconds() * 1000


async def stock_test2(request):
    wp = jp.WebPage(highcharts_theme="grid")
    ticker = request.query_params.get("ticker", "MSFT").upper()
    if ticker not in ["AAPL", "IBM", "INTC", "MSFT"]:
        ticker = "MSFT"
    data = await jp.JustPy.loop.run_in_executor(
        None, pd.read_csv, f"https://elimintz.github.io/stocks/{ticker}.csv"
    )
    chart = jp.HighStock(
        a=wp,
        classes="m-1 p-2 border w-10/12",
        options=chart_dict,
        style="height: 600px",
    )
    o = chart.options
    o.title.text = f"{ticker} Historical Prices"
    x = list(data["Date"].map(convert_date2))
    o.series[0].data = list(
        zip(x, data["Open"], data["High"], data["Low"], data["Close"])
    )
    o.series[0].name = ticker
    o.series[1].data = list(zip(x, data["Volume"]))
    return wp


# initialize the demo
from examples.basedemo import Demo
Demo("stock_test2", stock_test2)



# Justpy Tutorial demo animation_test from docs/reference/htmlcomponent.md
import justpy as jp
import random

input_classes = "m-2 bg-gray-200 appearance-none border-2 border-gray-200 rounded w-64 py-2 px-4 text-gray-700 focus:outline-none focus:bg-white focus:border-purple-500"


def animate(self, msg):
    self.d.delete_components()  # remove all components from d
    directions = ['Up', 'Down', 'Left', 'Right']
    html_entity = False
    for letter in self.text_to_animate.value:
        if letter == ' ':
            letter = '&nbsp;'
            html_entity = True
        jp.Div(animation=f'fadeIn{random.choice(directions)}', text=letter, html_entity=html_entity,
               classes='rounded-full bg-blue-500 text-white text-6xl', a=self.d)
        html_entity = False


def animation_test():
    wp = jp.WebPage()
    text_to_animate = jp.Input(a=wp, classes=input_classes, placeholder='Enter text to animate', value='Animation Demo!', input='return True')
    animate_btn = jp.Button(text='Animate!', click=animate, classes='w-32 m-2 bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded', a=wp)
    animate_btn.text_to_animate= text_to_animate
    d = jp.Div(classes='flex items-center justify-center', style='height: 500px', a=wp)
    animate_btn.d = d
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("animation_test",animation_test)



# Justpy Tutorial demo inner_html_test from docs/reference/htmlcomponent.md
import justpy as jp

def inner_html_test():
    wp = jp.WebPage()
    for i in range(1,11):
        jp.Div(inner_html=f'<span style="color: orange">{i}) Hello!</span>', a=wp, classes='m-2 p-2 text-3xl')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("inner_html_test",inner_html_test)



# Justpy Tutorial demo ajax_bar_example from docs/quasar_tutorial/quasar_components.md
import justpy as jp

async def start_bar(self, msg):
    wp = msg.page
    await wp.ajax_bar.run_method('start()', msg.websocket)

async def stop_bar(self, msg):
    wp = msg.page
    await wp.ajax_bar.run_method('stop()', msg.websocket)


def ajax_bar_example():
    wp = jp.QuasarPage()
    d = jp.Div(classes='q-pa-md', a=wp)
    # temp=False is important because this generates an id for the element that is required for run_method to work
    wp.ajax_bar = jp.QAjaxBar(position='bottom', color='accent', size='10px', skip_hijack=True, a=d, temp=False)
    btn_start = jp.QBtn(color='primary', label='Start Bar', a=d, click=start_bar, style='margin-right: 20px')
    btn_stop = jp.QBtn(color='primary', label='Stop Bar', a=d, click=stop_bar)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("ajax_bar_example",ajax_bar_example)



# Justpy Tutorial demo rating_test2 from docs/quasar_tutorial/QRating.md
import justpy as jp


def rating_test2():
    wp = jp.QuasarPage()
    wp.tailwind = True
    num_stars = 3
    r = jp.QRating(
        size="2em",
        max=num_stars,
        color="primary",
        classes="m-2 p-2",
        a=wp,
        value=2,
        debounce=0,
    )
    for i in range(1, num_stars + 1, 1):
        t = jp.QTooltip(text=f"{i} rating")
        r.add_scoped_slot(f"tip-{i}", t)
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("rating_test2", rating_test2)



# Justpy Tutorial demo hello_test4 from docs/tutorial/custom_components.md
import justpy as jp

class MyHello(jp.Hello):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.classes = 'm-1 p-1 text-6xl text-center text-red-500 bg-yellow-500 hover:bg-yellow-800 cursor-pointer'
        self.text = 'Much Better Hello! (click me)'

def hello_test4():
    wp = jp.WebPage()
    for i in range(5):
        MyHello(a=wp, counter=100)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("hello_test4",hello_test4)



"""
Created on 2022-08-24

@author: em
"""
from justpy import Div, Ul, Li, A


class Tabs(Div):

    tab_label_classes = "overflow-hidden cursor-pointer bg-white inline-block py-2 px-4 text-blue-500 hover:text-blue-800 font-semibold"
    tab_label_classes_selected = "overflow-hidden cursor-pointer bg-white inline-block border-l border-t border-r rounded-t py-2 px-4 text-blue-700 font-semibold"
    item_classes = "flex-shrink mr-1"
    item_classes_selected = "flex-shrink -mb-px mr-1"
    wrapper_style = "display: flex; position: absolute; width: 100%; height: 100%;  align-items: center; justify-content: center; background-color: #fff;"

    def __init__(self, **kwargs):

        self.tabs = []  # list of {'id': id, 'label': label, 'content': content}
        self.value = (
            None  # The value of the tabs component is the id of the selected tab
        )
        self.content_height = 500
        self.last_rendered_value = None
        self.animation = False
        self.animation_next = "slideInRight"
        self.animation_prev = "slideOutLeft"
        self.animation_speed = "faster"  # '' | 'slow' | 'slower' | 'fast'  | 'faster'

        super().__init__(**kwargs)

        self.tab_list = Ul(classes="flex flex-wrap border-b", a=self)
        self.content_div = Div(a=self)
        self.delete_list = []

    def __setattr__(self, key, value):
        if key == "value":
            try:
                self.previous_value = self.value
            except:
                pass
        self.__dict__[key] = value

    def add_tab(self, id, label, content):
        self.tabs.append({"id": id, "label": label, "content": content})
        if not self.value:
            self.value = id

    def get_tab_by_id(self, id):
        for tab in self.tabs:
            if tab["id"] == id:
                return tab
        return None

    def set_content_div(self, tab):
        self.content_div.add(tab["content"])
        self.content_div.set_classes("relative overflow-hidden border")
        self.content_div.style = f"height: {self.content_height}px"

    def set_content_animate(self, tab):
        self.wrapper_div_classes = (
            self.animation_speed
        )  # Component in this will be centered

        if self.previous_value:
            self.wrapper_div = Div(
                classes=self.wrapper_div_classes,
                animation=self.animation_next,
                temp=True,
                style=f"{self.wrapper_style} z-index: 50;",
                a=self.content_div,
            )
            self.wrapper_div.add(tab["content"])
            self.wrapper_div = Div(
                classes=self.wrapper_div_classes,
                animation=self.animation_prev,
                temp=True,
                style=f"{self.wrapper_style} z-index: 0;",
                a=self.content_div,
            )
            self.wrapper_div.add(self.get_tab_by_id(self.previous_value)["content"])
        else:
            self.wrapper_div = Div(
                classes=self.wrapper_div_classes,
                temp=True,
                a=self.content_div,
                style=self.wrapper_style,
            )
            self.wrapper_div.add(tab["content"])

        self.content_div.set_classes("relative overflow-hidden border")
        self.content_div.style = f"height: {self.content_height}px"

    def model_update(self):
        val = self.model[0].data[self.model[1]]
        if self.get_tab_by_id(val):
            self.value = val

    def delete(self):
        for c in self.delete_list:
            c.delete_flag = True
            c.delete()
            c.needs_deletion = False

        if self.delete_flag:
            for tab in self.tabs:
                tab["content"].delete()
                tab["content"] = None
        super().delete()

    @staticmethod
    async def tab_click(self, msg):
        if self.tabs.value != self.tab_id:
            previous_tab = self.tabs.value
            self.tabs.value = self.tab_id
            if hasattr(self.tabs, "model"):
                self.tabs.model[0].data[self.tabs.model[1]] = self.tabs.value
            # Run change if it exists
            if self.tabs.has_event_function("change"):
                msg.previous_tab = previous_tab
                msg.new_tab = self.tabs.value
                msg.id = self.tabs.id
                msg.value = self.tabs.value
                msg.class_name = self.tabs.__class__.__name__
                return await self.tabs.run_event_function("change", msg)
        else:
            return True  # No need to update page

    def convert_object_to_dict(self):
        if hasattr(self, "model"):
            self.model_update()
        self.set_classes("flex flex-col")
        self.tab_list.delete_components()
        self.content_div.components = []
        for tab in self.tabs:
            if tab["id"] != self.value:
                tab_li = Li(a=self.tab_list, classes=self.item_classes)
                li_item = A(
                    text=tab["label"],
                    classes=self.tab_label_classes,
                    a=tab_li,
                    delete_flag=False,
                )
                self.delete_list.append(li_item)
            else:
                tab_li = Li(a=self.tab_list, classes=self.item_classes_selected)
                li_item = A(
                    text=tab["label"],
                    classes=self.tab_label_classes_selected,
                    a=tab_li,
                    delete_flag=False,
                )
                self.delete_list.append(li_item)
                if self.animation and (self.value != self.last_rendered_value):
                    self.set_content_animate(tab)
                else:
                    self.set_content_div(tab)
            li_item.tab_id = tab["id"]
            li_item.tabs = self
            li_item.on("click", self.tab_click)
        self.last_rendered_value = self.value
        d = super().convert_object_to_dict()

        return d


class TabsPills(Tabs):
    tab_label_classes = "cursor-pointer inline-block border border-white rounded hover:border-gray-200 text-blue-500 hover:bg-gray-200 py-1 px-3"
    tab_label_classes_selected = "cursor-pointer inline-block border border-blue-500 rounded py-1 px-3 bg-blue-500 text-white"
    item_classes = "flex-shrink mr-3"
    item_classes_selected = "flex-shrink -mb-px mr-3"



"""
Created on 29.08.2022

@author: wf
"""
from tests.basetest import Basetest
from jpcore.download import Download
import os


class TestDownlad(Basetest):
    """
    test the download utility class
    """

    def testDownloadChinookDb(self):
        """
        see https://justpy.io/grids_tutorial/database/
        """
        dbname = "chinook.db"
        url = f"https://elimintz.github.io/{dbname}"
        os.makedirs(Download.get_cache_path(), exist_ok=True)
        filePath = f"{Download.get_cache_path()}/{dbname}"
        Download.download_file(url, dbname, target_directory=Download.get_cache_path())
        self.assertTrue(os.path.isfile(filePath))



"""
Created on 2021-03-20

A basic sanity test

@author: TalAmuyal

"""
from tests.basetest import Basetest
import io
from contextlib import redirect_stdout
import pprint
import sys

class TestImportability(Basetest):
    """
    Tests importability of justpy
    """

    def test_importability(self):
        """
        test the import
        """
        # this actually already runs some code
        with io.StringIO() as outputBuf, redirect_stdout(outputBuf):
            import justpy

            assert justpy
            outputText = outputBuf.getvalue()
        debug = self.debug
        jp_refcount=sys.getrefcount("justpy")
        debug=True
        if debug:
            print(f"{jp_refcount}:{outputText}")
        if jp_refcount<=16:
            self.assertTrue("Module directory" in outputText)
            self.assertTrue("Application directory" in outputText)
        # https://stackoverflow.com/questions/34861137/list-all-the-elements-in-a-python-namespace
        names = dir(justpy)
        if debug:
            pprint.pprint(names)
            print(f"{len(names)} names found")
        # @TODO - this should be shorter in the future
        self.assertTrue(len(names) > 400)
        pass



# Justpy Tutorial demo form_test from docs/tutorial/form.md
import justpy as jp
button_classes = 'bg-transparent hover:bg-blue-500 text-blue-700 font-semibold hover:text-white py-2 px-4 border border-blue-500 hover:border-transparent rounded m-2'
input_classes = 'border m-2 p-2'

session_data = {}

def form_test():
    wp = jp.WebPage()
    wp.display_url = '/fill_form'
    
    form1 = jp.Form(a=wp, classes='border m-1 p-1 w-64')

    user_label = jp.Label(text='User Name', classes='block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2', a=form1)
    in1 = jp.Input(placeholder='User Name', a=form1, classes='form-input')
    user_label.for_component = in1

    password_label = jp.Label(classes='block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2 mt-2', a=form1)
    jp.Div(text='Password', classes='block uppercase tracking-wide text-gray-700 text-xs font-bold mb-2', a=password_label)
    jp.Input(placeholder='Password', a=password_label, classes='form-input', type='password')

    check_label = jp.Label(classes='text-sm block', a=form1)
    jp.Input(type='checkbox', a=check_label, classes='form-checkbox text-blue-500')
    jp.Span(text='Send me stuff', a=check_label, classes= 'ml-2')
    submit_button = jp.Input(value='Submit Form', type='submit', a=form1, classes=button_classes)

    def submit_form(self, msg):
        print(msg)
        msg.page.redirect = '/form_submitted'
        session_data[msg.session_id] = msg.form_data

    form1.on('submit', submit_form)

    return wp

@jp.SetRoute('/form_submitted')
def form_submitted(request):
    wp = jp.WebPage()
    wp.display_url = '/thanks'
    jp.Div(text='Thank you for submitting the form', a=wp, classes='text-xl m-2 p-2')
    for field in session_data[request.session_id]:
        if field.type in ['text', 'password']:
            jp.Div(text=f'{field.placeholder}:  {field.value}', a=wp, classes='text-lg m-1 p-1')
        elif field.type == 'checkbox' and field.checked:
            jp.Div(text='We will send you stuff', a=wp, classes='text-lg m-1 p-1')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("form_test",form_test)



# Justpy Tutorial demo stock_test from docs/quasar_tutorial/QInput.md
import justpy as jp
from pandas_datareader import data as pdr
import datetime
import functools

epoch = datetime.datetime(1970, 1, 1)
grouping_units = [['week', [1]], ['month', [1, 2, 3, 4, 6]]]

chart_dict = {
    'rangeSelector': {'selected': 1},
    'yAxis': [
        {'labels': {'align': 'right', 'x': -3}, 'title': {'text': 'OHLC'}, 'height': '60%', 'lineWidth': 2, 'resize': {'enabled': True}},
        {'labels': {'align': 'right', 'x': -3}, 'title': {'text': 'Volume'}, 'top': '65%', 'height': '35%', 'offset': 0, 'lineWidth': 2}
    ],
    'tooltip': {'split': True},
    'series': [
        {'type': 'candlestick', 'tooltip': {'valueDecimals': 2}, 'dataGrouping': {'units': grouping_units}},
        {'type': 'column', 'name': 'Volume', 'yAxis': 1, 'dataGrouping': {'units': grouping_units}}
    ]
}


class QInputDate(jp.QInput):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        date_slot = jp.QIcon(name='event', classes='cursor-pointer')
        c2 = jp.QPopupProxy(transition_show='scale', transition_hide='scale', a=date_slot)
        self.date = jp.QDate(mask='YYYY-MM-DD', name='date', a=c2)

        self.date.parent = self
        self.date.value = self.value
        self.append_slot = date_slot
        self.date.on('input', self.date_time_change)
        self.on('input', self.input_change)
        self.proxy = c2

    @staticmethod
    async def date_time_change(self, msg):
        self.parent.value = self.value
        self.parent.date.value = self.value
        await self.parent.proxy.run_method('hide()', msg.websocket)

    @staticmethod
    def input_change(self, msg):
        self.date.value = self.value


def convert_date(date_string):
    date = datetime.datetime.strptime(str(date_string), '%Y-%m-%d')
    return (date - epoch).total_seconds()*1000


async def get_chart(self, msg):
    self.loading = True
    await msg.page.update()
    data = await jp.JustPy.loop.run_in_executor(None, functools.partial(pdr.DataReader, data_source='yahoo', start=self.start_date.value, end=self.end_date.value), self.ticker.value)
    data['Date'] = data.index.astype(str)
    chart = jp.HighStock(a=msg.page, classes='q-ma-md', options=chart_dict, style='height: 600px')
    o = chart.options
    ticker = self.ticker.value
    o.title.text = f'{ticker} Historical Prices'
    x = list(data['Date'].map(convert_date))
    o.series[0].data = list(zip(x, data['Open'], data['High'], data['Low'], data['Close']))
    o.series[0].name = ticker
    o.series[1].data = list(zip(x, data['Volume']))
    self.loading = False


async def stock_test(request):
    wp = jp.QuasarPage(highcharts_theme='grid')
    d = jp.Div(classes="q-ma-md q-gutter-md row", a=wp)
    ticker = jp.QInput(label='Ticker', a=d, value='MSFT')
    start_date = QInputDate(a=d, label='Start Date', standout=True, value='2007-01-01')
    end_date = QInputDate(a=d, label='End Date', standout=True, value='2019-12-31')
    b = jp.QBtn(label='Get Chart', a=d, start_date=start_date, end_date=end_date, ticker=ticker, click=get_chart, loading=False)
    return wp



# initialize the demo
from  examples.basedemo import Demo
Demo ("stock_test",stock_test)



# Justpy Tutorial demo event_propagates from docs/reference/htmlcomponent.md
import justpy as jp

def event_propagates():
    wp = jp.WebPage()
    main_div = jp.Div(classes='flex flex-wrap m-2 p-2 ', a=wp, click='self.text="main div clicked"')
    for i in range(1,10):
        jp.Div(text=f'Div {i}', a=main_div, classes='m-2 p-2 text-xl text-white bg-blue-500', click='self.text="clicked"')
    return wp

@jp.SetRoute('/no_propagation')
def event_does_not_propagate():
    wp = jp.WebPage()
    main_div = jp.Div(classes='flex flex-wrap m-2 p-2 ', a=wp, click='self.text="main div clicked"')
    for i in range(1,10):
        jp.Div(text=f'Div {i}', a=main_div, classes='m-2 p-2 text-xl text-white bg-blue-500',
               event_propagation=False, click='self.text="clicked"')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("event_propagates",event_propagates)



# Justpy Tutorial demo focus_test_input from docs/tutorial/input.md
import justpy as jp

def my_blur(self, msg):
        self.set_focus = False

def key_down(self, msg):
    # print(msg.key_data)
    key = msg.key_data.key
    if key=='Escape':
        self.value=''
        return
    if key=='Enter':
        self.set_focus = False
        try:
            next_to_focus = self.input_list[self.num + 1]
        except:
            next_to_focus = self.input_list[0]
        next_to_focus.set_focus = True
        return
    return True  # Don't update the page


def focus_test_input():
    wp = jp.WebPage()
    d = jp.Div(classes='flex flex-col  m-2', a=wp, style='width: 600 px')
    input_list = []
    number_of_fields = 5
    for i in range(1, number_of_fields + 1):
        label = jp.Label( a=d, classes='m-2 p-2')
        jp.Span(text=f'Field {i}', a=label)
        in1 = jp.Input(classes=jp.Styles.input_classes, placeholder=f'{i} Type here', a=label, keydown=key_down, spellcheck="false")
        in1.on('blur', my_blur)
        in1.input_list = input_list
        in1.num = i - 1
        input_list.append(in1)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("focus_test_input",focus_test_input)



import justpy as jp
import pandas as pd

wm_df = pd.read_csv("https://elimintz.github.io/women_majors.csv").round(2)


async def select_all_rows(self, msg):
    await self.grid.run_api("selectAll()", msg.page)


async def deselect_rows(self, msg):
    await self.grid.run_api("deselectAll()", msg.page)


async def resetFilters(self, msg):
    await self.grid.run_api("setFilterModel()", msg.page)


async def restoreFilters(self, msg):
    # savedFilterValues = msg.page.filterValues
    await self.grid.run_api(
        "setFilterModel({year: {type: 'lessThan',filter: '1980'}})", msg.page
    )


def row_selected(self, msg):
    wp = msg.page
    if msg.selected:
        wp.selected_rows[msg.rowIndex] = msg.data
    else:
        wp.selected_rows.pop(msg.rowIndex)


def downloadRow(self, msg):
    wp = msg.page
    wp.resultSelect.text = wp.selected_rows.values()


def grid_test():
    wp = jp.QuasarPage(dark=False)

    wp.selected_rows = {}

    grid = wm_df.jp.ag_grid(a=wp)
    grid.options.pagination = True
    grid.options.paginationAutoPageSize = True
    grid.options.columnDefs[0].checkboxSelection = True
    grid.options.columnDefs[0].headerCheckboxSelection = True
    grid.options.columnDefs[0].headerCheckboxSelectionFilteredOnly = True
    grid.options.columnDefs[1].hide = True
    # grid.options.columnDefs[1].floatingFilter = True

    # grid.options.defaultColDef.filter = True
    grid.options.defaultColDef.floatingFilter = True
    grid.options.defaultColDef.enableValue = True
    grid.options.defaultColDef.editable = True

    grid.options.rowSelection = "multiple"
    grid.options.sideBar = True
    grid.on("rowSelected", row_selected)

    d = jp.Div(classes="q-pa-md q-gutter-sm", a=wp)
    jp.QButton(label="Download", color="primary", a=d, click=downloadRow)
    buttonResetFilter = jp.QButton(
        label="Reset filter", color="primary", a=d, click=resetFilters
    )
    buttonResetFilter.grid = grid
    restoreRestoreFilter = jp.QButton(
        label="Restore filter", color="primary", a=d, click=restoreFilters
    )
    restoreRestoreFilter.grid = grid
    wp.resultSelect = jp.Div(
        classes="q-pa-md q-gutter-sm", a=wp, text="The result will be displayed here"
    )

    return wp


from examples.basedemo import Demo

Demo("Stackoverflow question 73497028 test", grid_test)



# Justpy Tutorial demo html_comps6 from docs/tutorial/html_components.md
import justpy as jp

def html_comps6():
    wp = jp.WebPage()
    for degree in range(0, 361, 10):
        image = jp.Img(src='https://www.python.org/static/community_logos/python-powered-h-140x182.png', a=wp)
        image.classes = 'm-4 p-4 inline-block'
        image.style = f'transform: rotate({degree}deg)'
        image.height = 100
        image.width = 100
        image.degree = degree

        def straighten1(self, msg):
            self.style = f'transform: rotate(0deg)'

        def rotate_back1(self, msg):
            self.style = f'transform: rotate({self.degree}deg)'

        def no_rotate1(self, msg):
            self.degree = 0
            self.set_class('bg-red-200')

        image.on('mouseenter', straighten1)
        image.on('mouseleave', rotate_back1)
        image.on('click', no_rotate1)

    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("html_comps6",html_comps6)



# Justpy Tutorial demo svg_demo3 from docs/tutorial/svg_components.md
import justpy as jp
# Example based on https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill

svg_html = """
<svg viewBox="0 0 300 100" xmlns="http://www.w3.org/2000/svg" width=300 height=100>
  <!-- Simple color fill -->
  <circle cx="50" cy="50" r="40" fill="pink" name="simple_circle"/>

  <!-- Fill circle with a gradient -->
  <defs>
    <radialGradient id="myGradient" name="gradient">
      <stop offset="0%"   stop-color="pink" name="stop"/>
      <stop offset="100%" stop-color="black" />
    </radialGradient>
  </defs>

  <circle cx="150" cy="50" r="40" fill="url(#myGradient)" name="gradient_circle"/>

  <!--
  Keeping the final state of an animated circle
  which is a circle with a radius of 40.
  -->
  <circle cx="250" cy="50" r="20">
    <animate attributeType="XML"
             attributeName="r"
             from="0" to="40" dur="5s"
             fill="freeze" />
  </circle>
</svg>
"""

def circle_click(self, msg):
    if self.stop.stop_color == 'orange':
        self.stop.stop_color = self.original_color
    else:
        self.stop.stop_color = 'orange'

def svg_demo3():
    wp = jp.WebPage()
    colors = ['pink', 'red', 'blue', 'green', 'teal', 'yellow']
    for color in colors:
        g = jp.parse_html(svg_html.replace('id="myGradient"', f'id="{color}"'), a=wp)
        g.name_dict["gradient_circle"].original_color = color
        gradient = g.name_dict["gradient"]
        g.name_dict["gradient_circle"].fill = f'url(#{gradient.id})'
        g.name_dict["gradient_circle"].stop = g.name_dict["stop"]
        g.name_dict["gradient_circle"].on('click', circle_click)
        g.name_dict["stop"].stop_color = color
        g.name_dict["simple_circle"].fill = color
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("svg_demo3",svg_demo3)



# Justpy Tutorial demo classes_test from docs/reference/htmlcomponent.md
import justpy as jp

def classes_test():
    wp = jp.WebPage()
    d = jp.Div(text='Classes Example', a=wp)
    # Assign Tailwind classes to d
    d.classes = 'text-5xl text-white bg-blue-500 hover:bg-blue-700 m-2 p-2 w-64'
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("classes_test",classes_test)



# Justpy Tutorial demo grid_test18 from docs/grids_tutorial/pandas.md
import justpy as jp
import pandas as pd

wm_df = pd.read_csv('https://elimintz.github.io/women_majors.csv').round(2)

def grid_test18():
    wp = jp.WebPage()
    grid = wm_df.jp.ag_grid(a=wp)
    grid.options.pagination = True
    grid.options.paginationAutoPageSize = True
    grid.options.columnDefs[0].cellClass = ['text-white', 'bg-blue-500', 'hover:bg-blue-200']
    for col_def in grid.options.columnDefs[1:]:
        col_def.cellClassRules = {
            'font-bold': 'x < 20',
            'bg-red-300': 'x < 20',
            'bg-yellow-300': 'x >= 20 && x < 50',
            'bg-green-300': 'x >= 50'
        }
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("grid_test18",grid_test18)



# Justpy Tutorial demo hello_world from docs/tutorial/getting_started.md
import justpy as jp


def hello_world():
    wp = jp.WebPage()
    jp.Hello(a=wp)
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("hello_world", hello_world)



'''
Created on 2022-09-16

@author: wf
'''
from tests.basetest import Basetest
from jpcore.tutorial import TutorialManager
from jpcore.demostarter import Demostarter
from jpcore.example import ExampleSource

class TestTutorial(Basetest):
    """
    test synchronization between Tutorial and examples
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.tm=TutorialManager()
        self.ds=Demostarter()
    
    def check_tutorial(self,tm,ds,debug=False):
        """
        check the tutorial against the demo starter
        """
        if debug:
            print(f"found {len(tm.tutorials)} tutorials with {tm.total_examples} examples ({tm.total_lines} lines)")
        for i,tutorial in enumerate(tm.tutorials.values()):
            print (f"{i+1:3}:{tutorial.name} ({len(tutorial.lines):4} lines)")
            for j,example in enumerate(tutorial.examples.values()):
                print (f"  {j+1:2}:{example.name} ({example.header}) - {example.example_source.url}")
        header_missing=0
        html_used=0
        demo_missing=0
        for i,tutorial in enumerate(tm.tutorials.values()):
            for j,example in enumerate(tutorial.examples.values()):
                if example.option is not None:
                    if debug:
                        print(f"{tutorial.name} {example.name} option {example.option} used")
                        pass
                if not example.name in ds.demos_by_name:
                    demo_missing+=1
                    if debug:
                        print(f"❌ docs/{tutorial.name} {example.name} - demo missing")
                if example.header is None:
                    header_missing+=1
                    if debug:
                        print(f"❌ docs/{tutorial.name} {example.name} - header missing")
                elif "<" in example.header:
                    if debug:
                        print(f"❌ docs/{tutorial.name} {example.name} {example.header} - contains html markup")
                        html_used+=1
        if debug:
            print(f"{demo_missing} demos missing {header_missing} headers missing {html_used} x html used")
        self.assertEqual(0,header_missing+html_used)
        for demo in ds.demos:
            if not demo.name in tm.examples_by_name:
                if demo.example_source.source_type=="tutorial":
                    if debug:
                        print(f"❌ {demo.name} not linked to tutorial examples")
                
    def test_tutorial_manager(self):
        """
        test the tutorial manager
        """
        debug=True
        self.check_tutorial(self.tm,self.ds,debug=debug)
        if debug:
            print(f"found {len(self.tm.tutorials)} tutorial files")
        self.assertTrue(len(self.tm.tutorials)>=70)
        
    def show_problems(self,problems):
        """
        show the given list of problems
        """ 
        for i,problem in enumerate(problems):
            print(f"  {i+1:2}:{problem}")
        
    def test_update_from_tutorial(self):
        """
        test updating a demo from the tutorial source
        """
        debug=False
        for demo_name in ["stock_test2"]:
            demo=self.ds.demos_by_name[demo_name]
            tutorial_example=self.tm.examples_by_name[demo_name]
            tutorial_source=tutorial_example.example_source
            target_path=f"/tmp/{demo_name}.py"
            demo.update_from_tutorial(tutorial_source=tutorial_source,target_path=target_path)
            update_source=ExampleSource(description=f"test for {demo_name}.py") 
            update_source.read_source(target_path)
            if debug:
                for i,line in enumerate(update_source.lines):
                    print (f"{i+1:3} {line}")
            problems=demo.same_as_tutorial(update_source,debug=debug) 
            if debug:
                self.show_problems(problems)       
            self.assertEqual(0,len(problems))
        
    def test_tutorial_diff(self):
        """
        test the tutorial examples for matching with the tutorial source code
        """
        debug=True
        failed_checks=0
        for demo in self.ds.demos:
            if demo.name in self.tm.examples_by_name:
                tutorial_example=self.tm.examples_by_name[demo.name]
                tutorial_source=tutorial_example.example_source
                problems=demo.same_as_tutorial(tutorial_source,debug=False)
                if len(problems)>0:
                    failed_checks+=1
                    if debug:
                        print (demo.pymodule_file)
                        print (tutorial_example.tutorial)
                        print (tutorial_source.url)
                        self.show_problems(problems)
        if debug:
            print(f"❌ {failed_checks}/{len(self.ds.demos)} examples are not in sync with the tutorial content")
        #self.assertEqual(0,failed_checks)
        self.assertTrue(failed_checks<=16)


# This is the "dogs" example for justpy
# see https://github.com/justpy-org/justpy/blob/master/examples/dogs.py
from justpy import *

# https://dog.ceo/api/breeds/list/all    dict of all breeds under

breeds = [
    "affenpinscher",
    "african",
    "airedale",
    "akita",
    "appenzeller",
    "basenji",
    "beagle",
    "bluetick",
    "borzoi",
    "bouvier",
    "boxer",
    "brabancon",
    "briard",
    "bullterrier-staffordshire",
    "cairn",
    "cattledog-australian",
    "chihuahua",
    "chow",
    "clumber",
    "cockapoo",
    "collie-border",
    "coonhound",
    "corgi-cardigan",
    "cotondetulear",
    "dachshund",
    "dalmatian",
    "deerhound-scottish",
    "dhole",
    "dingo",
    "doberman",
    "elkhound-norwegian",
    "entlebucher",
    "eskimo",
    "frise-bichon",
    "germanshepherd",
    "greyhound-italian",
    "groenendael",
    "hound-blood",
    "hound-english",
    "hound-ibizan",
    "hound-walker",
    "husky",
    "keeshond",
    "kelpie",
    "komondor",
    "kuvasz",
    "labrador",
    "leonberg",
    "lhasa",
    "malamute",
    "malinois",
    "maltese",
    "mastiff-bull",
    "mastiff-tibetan",
    "mexicanhairless",
    "mix",
    "mountain-bernese",
    "mountain-swiss",
    "newfoundland",
    "otterhound",
    "papillon",
    "pekinese",
    "pembroke",
    "pinscher-miniature",
    "pointer-german",
    "pomeranian",
    "pug",
    "puggle",
    "pyrenees",
    "redbone",
    "retriever-chesapeake",
    "retriever-curly",
    "retriever-flatcoated",
    "retriever-golden",
    "ridgeback-rhodesian",
    "rottweiler",
    "saluki",
    "samoyed",
    "schipperke",
    "schnauzer-giant",
    "schnauzer-miniature",
    "setter-english",
    "setter-gordon",
    "setter-irish",
    "sheepdog-english",
    "sheepdog-shetland",
    "shiba",
    "shihtzu",
    "spaniel-blenheim",
    "spaniel-brittany",
    "spaniel-cocker",
    "spaniel-irish",
    "spaniel-japanese",
    "spaniel-sussex",
    "spaniel-welsh",
    "springer-english",
    "stbernard",
    "terrier-american",
    "terrier-australian",
    "terrier-bedlington",
    "terrier-border",
    "terrier-dandie",
    "terrier-fox",
    "terrier-irish",
    "terrier-kerryblue",
    "terrier-lakeland",
    "terrier-norfolk",
    "terrier-norwich",
    "terrier-patterdale",
    "terrier-russell",
    "terrier-scottish",
    "terrier-sealyham",
    "terrier-silky",
    "terrier-tibetan",
    "terrier-toy",
    "terrier-westhighland",
    "terrier-wheaten",
    "terrier-yorkshire",
    "vizsla",
    "weimaraner",
    "whippet",
    "wolfhound-irish",
]


async def dog_test(_request):
    """
    create a reactive webpage for dog pictures taken from
    """
    wp = QuasarPage()
    wp.body_style = "overflow: hidden"
    bp = parse_html(
        """
    <div class="q-pa-md">
    <q-layout view="HHH lpr Fff" container  style="height: 95vh" class="shadow-2 rounded-borders">
        <q-header reveal class="bg-purple">
            <q-toolbar name="toolbar">
                <q-btn flat round dense icon="menu"/>
                <q-toolbar-title>Dog Pictures</q-toolbar-title>
                <q-btn glossy label="Next Picture" classes="q-mr-sm" name="next_picture"></q-btn>
            </q-toolbar>
        </q-header>
        <q-drawer name="drawer" width=300 breakpoint=700 show-if-above elevated content-class="bg-white text-blue">
            <q-scroll-area class="fit">
                <div class="q-pa-sm">
                    <q-list name="thumbnail_list">
                    </q-list>
                </div>
            </q-scroll-area>
        </q-drawer>
        <q-footer class="bg-purple text-white">
            <div class="text-white q-pa-xs" style="height: 30px;" name="footer"></div>
        </q-footer>
        <q-page-container style="overflow: hidden;">
            <q-page padding name="main_page" style="overflow: hidden;">
                    <div style="height: 400px; width: 600px">
                        <q-img src="https://images.dog.ceo/breeds/papillon/n02086910_6087.jpg" class="cursor-pointer" name="image" >
                       </div>
                       <q-tooltip content-class="bg-purple" content-style="font-size: 13px" transition-show="scale" transition-hide="scale" anchor="top middle"  name="tooltip">
          Click image for next picture
        </q-tooltip>
                    </q-img>
            </q-page>
        </q-page-container>
    </q-layout>
</div>
    """,
        a=wp,
    )
    main_image = bp.name_dict["image"]
    l = Link(
        href="https://quasar.dev", text="Quasar", target="_blank", classes="text-white"
    )
    bp.name_dict["footer"].add(l)
    bp.name_dict["tooltip"].disable_events = True
    main_image.breed = "papillon"
    breed_select = QBtnDropdown(
        auto_close=True,
        split=False,
        glossy=True,
        label="Select Breed",
        icon="fas fa-dog",
        a=bp.name_dict["toolbar"],
    )
    breed_list = QList(separator=True, dense=True, a=breed_select)

    async def change_breed(self, msg):
        main_image.breed = self.breed
        await change_pic(main_image, msg)
        breed_select.label = self.breed

    for breed in breeds:
        breed_item_html = f"""<q-item clickable v-close-menu v-ripple>
                            <q-item-section >
                                <q-item-label>{breed}</q-item-label>
                            </q-item-section>
                        </q-item>"""
        # breed_item = parse_html(breed_item_html).first()
        breed_item = parse_html(breed_item_html)
        breed_item.breed = breed
        breed_item.on("click", change_breed)
        breed_list.add(breed_item)

    def add_thumbnail(self, _msg):
        """
        add a thumbnail
        """
        list_item_html = f"""<q-item clickable v-ripple>
                            <q-item-section thumbnail>
                                <img src="{self.src}"/>
                            </q-item-section>
                            <q-item-section>{self.breed}</q-item-section>
                            <q-item-section> <q-btn class="gt-xs text-grey-8" size="12px" flat dense round icon="delete" name="delete"/></q-item-section>
                        </q-item>"""
        list_item = parse_html(list_item_html)
        list_item.breed = self.breed
        list_item.src = self.src

        list_item.name_dict["delete"].list = bp.name_dict["thumbnail_list"]
        list_item.name_dict["delete"].list_item = list_item
        bp.name_dict["thumbnail_list"].add(list_item)

        def display_thumbnail(self, _msg):
            """
            display a thumbnail
            """
            main_image.src = self.src

        list_item.on("mouseenter", display_thumbnail)

        def delete_list_item(self, _msg):
            """
            delete a list item
            """
            self.list.remove(self.list_item)

        list_item.name_dict["delete"].on("click", delete_list_item)

    async def change_pic(self, msg):
        """
        change the pictures
        """
        # https://dog.ceo/api/breed/bulldog/french/images/random
        if "-" in self.breed:
            b = self.breed.split("-")
            r = await get(f"https://dog.ceo/api/breed/{b[0]}/{b[1]}/images/random")
        else:
            r = await get(f"https://dog.ceo/api/breed/{self.breed}/images/random")
        self.src = r["message"]
        add_thumbnail(self, msg)

    async def next_pic(self, msg):
        """
        react on next picture clicked
        """
        return await change_pic(main_image, msg)

    # allow clicking an image
    main_image.on("click", change_pic)
    bp.name_dict["next_picture"].on("click", next_pic)

    # initial picture
    await change_pic(main_image, {})
    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("dogs demo", dog_test)



# This is the "black jack" example for justpy
# see https://github.com/justpy-org/justpy/blob/master/examples/blackjack.py

# Simplified blackjack game using https://deckofcardsapi.com/ API
# Not all rules are implemented

import justpy as jp
import asyncio

card_size = 0.5
card_width = int(226 * card_size)
card_height = int(card_width * 314 / 226)
button_classes = """
inline-flex items-center px-6 py-6 border border-gray-700 text-2xl leading-6 font-bold rounded-md text-gray-700 
bg-white hover:text-gray-500 focus:outline-none focus:border-blue-300 focus:shadow-outline-blue active:text-gray-800 
active:bg-gray-50 transition ease-in-out duration-150
"""
div_classes = "items-center px-6 py-6 border border-gray-700 text-3xl leading-6 font-bold rounded-md text-gray-700; bg-white "


class Card(jp.Img):
    """
    Card component: an extended image with predefined width and height and a transition
    """

    def __init__(self, **kwargs):
        """
        constructor
        """
        super().__init__(**kwargs)
        self.width = card_width
        self.height = card_height
        self.set_class("ml-2")

        self.transition = {
            "load": "transition origin-left ease-out duration-1000",
            "load_start": "transform scale-x-0",
            "load_end": "transform scale-x-100",
            "enter": "transition origin-left ease-out duration-1000",
            "enter_start": "transform scale-x-0",
            "enter_end": "transform scale-x-100",
        }


async def create_deck():
    """
    get a new deck
    """
    deck = await jp.get("https://deckofcardsapi.com/api/deck/new/shuffle/?deck_count=1")
    return deck["deck_id"]


async def deal(deck_id, count=1):
    """
    deal
    """
    try:
        cards = await jp.get(
            f"https://deckofcardsapi.com/api/deck/{deck_id}/draw/?count={count}"
        )
    except:
        print("error in deal")
    return cards


def hand_value(hand):
    """
    calculate the hand value
    """
    value = 0
    aces = 0
    for card in hand:
        if card.value == "ACE":
            aces += 1
            value += 1
        else:
            try:
                value += int(card.value)
            except:
                value += 10
    if aces and value < 12:
        value += 10
    return value


async def hit(self, msg):
    """
    react on hit button
    """
    wp = msg.page
    card_dict = jp.Dict(await deal(wp.deck_id))
    card = card_dict.cards[0]
    wp.player_hand.append(card)
    wp.player_div.add(Card(src=card.image))
    player_hand_value = hand_value(wp.player_hand)
    if player_hand_value < 22:
        wp.hand_value_div.text = f"Hand value: {player_hand_value}"
    else:
        wp.hand_value_div.text = f"YOU HAVE BUSTED, Hand value: {player_hand_value}"
        dealer_hand_value = hand_value(wp.dealer_hand)
        result_div = jp.Div(classes=div_classes, a=wp.outer_div)
        result_div.text = f"YOU LOST, Your hand: {player_hand_value}, Dealer's hand: {dealer_hand_value}"
        for btn in [wp.stand_btn, wp.hit_btn]:
            btn.disabled = True
            btn.set_classes("cursor-not-allowed bg-gray-200 opacity-50")
        wp.play_again_btn.remove_class("hidden")


async def stand(self, msg):
    """
    react on stand button
    """
    wp = msg.page
    # Show dealer card
    wp.card_back.set_class("hidden")
    wp.down_card.remove_class("hidden")
    for btn in [wp.stand_btn, wp.hit_btn]:
        btn.disabled = True
        btn.set_classes("cursor-not-allowed bg-gray-200 opacity-50")
    await wp.update()
    await asyncio.sleep(1.1)

    while True:
        dealer_hand_value = hand_value(wp.dealer_hand)
        if dealer_hand_value > 16:
            break
        card_dict = jp.Dict(await deal(wp.deck_id))
        card = card_dict.cards[0]
        wp.dealer_hand.append(card)
        wp.dealer_div.add(Card(src=card.image))
        await wp.update()
        await asyncio.sleep(1.1)
    player_hand_value = hand_value(wp.player_hand)
    result_div = jp.Div(classes=div_classes, a=wp.outer_div)
    if (dealer_hand_value > 21) or (dealer_hand_value < player_hand_value):
        result_div.text = f"YOU WON, Your hand: {player_hand_value}, Dealer's hand: {dealer_hand_value}"
    elif dealer_hand_value > player_hand_value:
        result_div.text = f"YOU LOST, Your hand: {player_hand_value}, Dealer's hand: {dealer_hand_value}"
    else:
        result_div.text = f"IT IS A DRAW, Your hand: {player_hand_value}, Dealer's hand: {dealer_hand_value}"

    wp.play_again_btn.remove_class("hidden")


async def play_again(self, msg):
    """
    react on play again button
    """
    wp = msg.page
    await wp.reload()


async def blackjack():
    """
    the async web page to serve
    """
    wp = jp.WebPage()
    wp.outer_div = jp.Div(
        classes="container mx-auto px-4 sm:px-6 lg:px-8 space-y-5", a=wp
    )
    wp.deck_id = await create_deck()
    # Deal initial four cards
    cards = jp.Dict(await deal(wp.deck_id, 4))
    wp.player_hand = [cards.cards[0], cards.cards[2]]
    wp.dealer_hand = [cards.cards[1], cards.cards[3]]
    jp.Div(
        text="Blackjack Demo",
        a=wp.outer_div,
        classes="m-2 p-4 text-3xl font-bold leading-7 text-gray-900  sm:leading-9 sm:truncate",
    )
    wp.dealer_div = jp.Div(classes="flex flex-wrap m-2", a=wp.outer_div)
    # Image of back of card
    wp.card_back = Card(
        src="https://raw.githubusercontent.com/elimintz/elimintz.github.io/master/card_back.png",
        a=wp.dealer_div,
    )
    wp.down_card = Card(src=wp.dealer_hand[0].image, a=wp.dealer_div, classes="hidden")
    Card(src=wp.dealer_hand[1].image, a=wp.dealer_div, style="transition-delay: 1000ms")
    wp.player_div = jp.Div(classes="flex flex-wrap m-2", a=wp.outer_div)
    for card in wp.player_hand:
        Card(src=card.image, a=wp.player_div, style="transition-delay: 2000ms")
    button_div = jp.Div(classes="flex m-2 space-x-6", a=wp.outer_div)
    wp.stand_btn = jp.Button(
        text="Stand", a=button_div, classes=button_classes, click=stand
    )
    wp.hit_btn = jp.Button(text="Hit", a=button_div, classes=button_classes, click=hit)
    wp.play_again_btn = jp.Button(
        text="Play Again", a=button_div, classes=button_classes, click=play_again
    )
    wp.play_again_btn.set_class("hidden")
    wp.hand_value_div = jp.Div(
        text=f"Hand value: {hand_value(wp.player_hand)}",
        a=wp.outer_div,
        classes="text-2xl",
    )
    return wp


from examples.basedemo import Demo

Demo("blackjack demo", blackjack)



# Justpy Tutorial demo todo_app3 from docs/blog/reactivity.md
import justpy as jp


class TodoList(jp.Ol):

    def __init__(self, **kwargs):
        self.todos = []
        super().__init__(**kwargs)

    def react(self, data):
        self.delete_components()
        for todo in self.todos:
            jp.Li(text=todo, a=self)


def todo_app3():
    wp = jp.WebPage(tailwind=False)
    todo_list = TodoList(a=wp, todos=["Buy groceries", "Learn JustPy", "Do errands"])
    task = jp.Input(placeholder="Enter task", a=wp)

    def add_task(self, msg):
        todo_list.todos.append(task.value)
        task.value = ""

    jp.Button(text="Add Task", a=wp, click=add_task, style="margin-left: 10px")

    return wp


# initialize the demo
from examples.basedemo import Demo

Demo("todo_app3", todo_app3)



# Justpy Tutorial demo link_demo1 from docs/tutorial/html_components.md
import justpy as jp

def link_demo1():
    wp = jp.WebPage()
    jp.A(text='Python Org', href='https://python.org', a=wp, classes='m-2 p-2 text-xl text-white bg-blue-500 hover:bg-blue-700')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("link_demo1",link_demo1)



# Justpy Tutorial demo create_chart_test5 from docs/charts_tutorial/creating_charts.md
import justpy as jp

# https://github.com/elimintz/elimintz.github.io/tree/master/charts
# Try https://127.0.0.1:8000/ + any one of the entries in charts
# For example: http://127.0.0.1:8000/bubbles
charts = ["org", "bubbles", "item", "timeline", "states", "browsers", "wheel"]


@jp.SetRoute("/{chart_name}")
async def create_chart_test5(request):
    wp = jp.WebPage()
    chart_name = request.path_params.get("chart_name", "item")   # Default chart is item
    if chart_name not in charts:
        chart_name = "item"
    chart_options = await jp.get(f"https://elimintz.github.io/charts/{chart_name}", "text")
    my_chart = jp.HighCharts(a=wp, classes="m-2 p-2 border w-3/4", options=chart_options)
    return wp

# initialize the demo
from examples.basedemo import Demo
Demo("create_chart_test5", create_chart_test5)



from justpy import Form, Input, Li, Ol, WebPage

# see https://github.com/elimintz/justpy/pull/401
def handle_submit(_c, msg):
    """
    handle submission of a multi upload
    """
    fl = msg.page.file_list
    fl.components.clear()
    for fd in msg.form_data:
        if "files" in fd:
            for f in fd["files"]:
                Li(text=f"File uploaded: {f['name']} of {len(f['file_content'])}", a=fl)


def multiupload():
    """
    show a multi upload
    """
    wp = WebPage()
    WebPage.tailwind = False
    f = Form(submit=handle_submit, a=wp)
    Input(name="f1", type="file", a=f)
    Input(name="f2", type="file", a=f)
    Input(type="submit", value="OK", a=f)
    wp.file_list = Ol(a=wp)
    return wp


from examples.basedemo import Demo

Demo("multi uploads demo", multiupload)



# Justpy Tutorial demo input_test8 from docs/quasar_tutorial/QInput.md
import justpy as jp

class PasswordWithToggle(jp.QInput):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.type = 'password'
        visibility_icon = jp.QIcon(name='visibility_off', classes='cursor-pointer')
        visibility_icon.password_input = self
        self.append_slot = visibility_icon
        visibility_icon.on('click', self.toggle_password)

    @staticmethod
    def toggle_password(self, msg):
        if self.name == 'visibility_off':
            self.name = 'visibility'
            self.password_input.type = 'text'
        else:
            self.name = 'visibility_off'
            self.password_input.type = 'password'


def input_test8(request):
    wp = jp.QuasarPage(data={'text': ''})
    c1 = jp.Div(classes='q-pa-md', a=wp)
    c2 = jp.Div(classes='q-gutter-md', style='max-width: 300px', a=c1)
    for i in range(1,6):
        PasswordWithToggle(filled=True,  type='password', a=c2, hint=f'Password with toggle #{i}')
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("input_test8",input_test8)



# Justpy Tutorial demo input_demo4 from docs/tutorial/input.md
import justpy as jp

input_classes = "m-2 bg-gray-200 border-2 border-gray-200 rounded w-64 py-2 px-4 text-gray-700 focus:outline-none focus:bg-white focus:border-purple-500"
p_classes = 'm-2 p-2 h-32 text-xl border-2'

def my_input3(self, msg):
    self.div.text = self.value

def input_demo4(request):
    wp = jp.WebPage()
    in1 = jp.Input(type='number', a=wp, classes=input_classes, placeholder='Please type here')
    in1.div = jp.Div(text='What you type will show up here', classes=p_classes, a=wp)
    in1.on('input', my_input3)
    return wp

# initialize the demo
from  examples.basedemo import Demo
Demo ("input_demo4",input_demo4)



"""
Created on 2022-09-16

@author: bapowell
"""
import justpy as jp
import pandas as pd
import uuid

df = pd.DataFrame({
    "col1": range(3),
    "col2": ['Abc', 'Def', 'Ghi'],
    "col3": [pd.Timestamp(year=2016, month=4, day=d+1) for d in range(3)],
    "col4": [uuid.uuid4() for _ in range(3)]
})

async def row_selected(self, msg):
    print(msg)
    if msg.selected:
        self.row_data_div.text = msg.data
        self.row_selected = msg.rowIndex
    elif self.row_selected == msg.rowIndex:
        self.row_data_div.text = ''

def manipulate_row_data(irow, icol, col_key, val):
    if isinstance(val, uuid.UUID):
        new_val = str(val)
    elif isinstance(val, pd.Timestamp):
        new_val = val.timestamp()  # convert to POSIX epoch seconds (float)
    else:
        new_val = val
    # print(irow, icol, col_key, new_val)
    return new_val

def grid_test_issue500():
    wp = jp.WebPage()
    row_data_div = jp.Div(a=wp, text="Replace me by selecting a row")
    grid = df.jp.ag_grid(a=wp, row_data_converter=manipulate_row_data)
    grid.row_data_div = row_data_div
    grid.on("rowSelected", row_selected)
    grid.options.columnDefs[0].checkboxSelection = True
    return wp


from examples.basedemo import Demo

Demo("Issue 500 aggrid uuid value", grid_test_issue500)



# Justpy Tutorial demo hello_world1 from docs/tutorial/basic_concepts.md
import justpy as jp


def hello_world1():
    wp = jp.WebPage()
    p = jp.P(text="Hello World!", a=wp)
    return wp


## JustPy Requests
# initialize the demo
from examples.basedemo import Demo

Demo("hello_world1", hello_world1)
